@@ -18,10 +18,12 @@ os_version() {
 }
 
 declare -a package_tree=(
+  bison
   cmake
   curl
   elfutils-libelf-devel
   findutils
+  flex
   gcc
   gcc-c++
   git
