@@ -330,19 +330,31 @@ prepare_cmake_options() {
   fi
 
   if [ -z "${ENABLE_SYSTEMD_JOURNAL}" ]; then
-      if check_for_module libsystemd; then
-          if check_for_module libelogind; then
-              ENABLE_SYSTEMD_JOURNAL=0
-          else
-              ENABLE_SYSTEMD_JOURNAL=1
-          fi
+    if check_for_module libsystemd; then
+      if check_for_module libelogind; then
+        ENABLE_SYSTEMD_JOURNAL=0
       else
-          ENABLE_SYSTEMD_JOURNAL=0
+        ENABLE_SYSTEMD_JOURNAL=1
       fi
+    else
+      ENABLE_SYSTEMD_JOURNAL=0
+    fi
   fi
 
   enable_feature PLUGIN_SYSTEMD_JOURNAL "${ENABLE_SYSTEMD_JOURNAL}"
 
+  if [ "${ENABLE_SYSTEMD_JOURNAL}" -eq 1 ] && [ -n "${USE_RUST_JOURNAL_FILE}" ]; then
+    enable_feature NETDATA_JOURNAL_FILE_READER 1
+  else
+    enable_feature NETDATA_JOURNAL_FILE_READER 0
+  fi
+
+  if check_for_module 'libsystemd >= 221'; then
+    enable_feature PLUGIN_SYSTEMD_UNITS 1
+  else
+    enable_feature PLUGIN_SYSTEMD_UNITS 0
+  fi
+
   if command -v cups-config >/dev/null 2>&1 || check_for_module libcups || check_for_module cups; then
     ENABLE_CUPS=1
   else
