@@ -23,7 +23,7 @@ DEFAULT_PLUGIN_PACKAGES=""
 REPOCONFIG_DEB_VERSION="5-1"
 REPOCONFIG_RPM_VERSION="5-1"
 START_TIME="$(date +%s)"
-STATIC_INSTALL_ARCHES="x86_64 armv7l armv6l aarch64 ppc64le"
+STATIC_INSTALL_ARCHES="x86_64 armv7l armv6l aarch64"
 
 # ======================================================================
 # Properly sort out inconsistencies in `$PATH` across distros
@@ -205,6 +205,7 @@ USAGE: kickstart.sh [options]
   --local-build-options            Specify additional options to pass to the installer code when building locally. Only valid if --build-only is also specified.
   --static-install-options         Specify additional options to pass to the static installer code. Only valid if --static-only is also specified.
   --offline-architecture           Limit an offline install source being prepared with --prepare-offline-install-source to only include the specified static build architecture.
+  --offline-install-source         Specify a folder to use as installation source when working offline. Use --prepare-offline-install-source first to set up this folder.
 
 The following options are mutually exclusive and specifiy special operations other than trying to install Netdata normally or update an existing install:
 
@@ -361,7 +362,7 @@ trap_handler() {
     0) printf >&2 "%s\n" "This is almost certainly the result of a bug. If you have time, please report it at ${AGENT_BUG_REPORT_URL}." ;;
     *)
       printf >&2 "%s\n" "This is probably a result of a transient issue on your system. Things should work correctly if you try again."
-      printf >&2 "%s\n" "If you continue to experience this issue, you can reacn out to us for support on:"
+      printf >&2 "%s\n" "If you continue to experience this issue, you can reach out to us for support on:"
       support_list
       ;;
   esac
