@@ -747,8 +747,14 @@ static int aclk_attempt_to_connect(mqtt_wss_client client)
 
         int ssl_flags = cloud_config_insecure_get() ? MQTT_WSS_SSL_ALLOW_SELF_SIGNED : MQTT_WSS_SSL_CERT_CHECK_FULL;
 
-        struct mqtt_wss_proxy proxy_conf = { .host = NULL, .port = 0, .username = NULL, .password = NULL, .type = MQTT_WSS_DIRECT };
-        aclk_set_proxy((char**)&proxy_conf.host, &proxy_conf.port, (char**)&proxy_conf.username, (char**)&proxy_conf.password, &proxy_conf.type);
+        struct mqtt_wss_proxy proxy_conf = { .host = NULL, .port = 0, .username = NULL, .password = NULL, .proxy_destination = NULL, .type = MQTT_WSS_DIRECT };
+        aclk_set_proxy(
+            (char **)&proxy_conf.host,
+            &proxy_conf.port,
+            (char **)&proxy_conf.username,
+            (char **)&proxy_conf.password,
+            (char **)&proxy_conf.proxy_destination,
+            &proxy_conf.type);
 
 #ifdef ACLK_DISABLE_CHALLENGE
         int mqtt_rc = mqtt_wss_connect(client, base_url.host, base_url.port, &mqtt_conn_params, ssl_flags, &proxy_conf);
@@ -836,7 +842,7 @@ void *aclk_main(void *ptr)
     worker_register_job_name(WORKER_ACLK_WAITING_TO_CONNECT, "conn wait");
 
     ACLK_PROXY_TYPE proxy_type;
-    aclk_get_proxy(&proxy_type);
+    aclk_get_proxy(&proxy_type, false);
     if (proxy_type == PROXY_TYPE_SOCKS5) {
         netdata_log_error("ACLK: SOCKS5 proxy is not supported by ACLK-NG yet.");
         static_thread->enabled = NETDATA_MAIN_THREAD_EXITED;
@@ -1056,6 +1062,9 @@ static void fill_alert_status_for_host(BUFFER *wb, RRDHOST *host)
         aclk_host_config->snapshot_count);
 }
 
+
+extern usec_t publish_latency;
+
 char *aclk_state(void)
 {
     BUFFER *wb = buffer_create(1024, &netdata_buffers_statistics.buffers_aclk);
@@ -1074,10 +1083,20 @@ char *aclk_state(void)
         buffer_strcat(wb, "No\n");
     else {
         const char *cloud_base_url = cloud_config_url_get();
-        buffer_sprintf(wb, "Yes\nClaimed Id: %s\nCloud URL: %s\n", claim_id.str, cloud_base_url ? cloud_base_url : "null");
+        char *aclk_proxy = (char *)aclk_get_proxy(NULL, true);
+        usec_t latency = __atomic_load_n(&publish_latency, __ATOMIC_RELAXED);
+        char latency_str[64];
+        duration_snprintf(latency_str, sizeof(latency_str), (int64_t) latency, "us", true);
+        buffer_sprintf(wb, "Yes\nClaimed Id: %s\nCloud URL: %s\nACLK Proxy: %s\nPublish Latency: %s\n", claim_id.str, cloud_base_url ? cloud_base_url : "null", aclk_proxy ? aclk_proxy : "none", latency_str);
     }
 
-    buffer_sprintf(wb, "Online: %s\nReconnect count: %d\nBanned By Cloud: %s\n", aclk_online() ? "Yes" : "No", aclk_connection_counter > 0 ? (aclk_connection_counter - 1) : 0, aclk_disable_runtime ? "Yes" : "No");
+    bool aclk_is_online = aclk_online();
+
+    struct mqtt_wss_stats aclk_stats;
+    if (aclk_is_online)
+        aclk_stats = aclk_statistics();
+
+    buffer_sprintf(wb, "Online: %s\nReconnect count: %d\nBanned By Cloud: %s\n", aclk_is_online ? "Yes" : "No", aclk_connection_counter > 0 ? (aclk_connection_counter - 1) : 0, aclk_disable_runtime ? "Yes" : "No");
     if (last_conn_time_mqtt && ((tmptr = localtime_r(&last_conn_time_mqtt, &tmbuf))) ) {
         char timebuf[26];
         strftime(timebuf, 26, "%Y-%m-%d %H:%M:%S", tmptr);
@@ -1099,8 +1118,9 @@ char *aclk_state(void)
         buffer_sprintf(wb, "Next Connection Attempt At: %s\nLast Backoff: %.3f", timebuf, last_backoff_value);
     }
 
-    if (aclk_online()) {
-        buffer_sprintf(wb, "Received Cloud MQTT Messages: %d\nMQTT Messages Confirmed by Remote Broker (PUBACKs): %d", aclk_rcvd_cloud_msgs, aclk_pubacks_per_conn);
+    if (aclk_is_online) {
+        buffer_sprintf(wb, "Received Cloud MQTT Messages: %d\nMQTT Messages Confirmed by Remote Broker (PUBACKs): %d\nPending PUBACKS: %d\n",
+                       aclk_rcvd_cloud_msgs, aclk_pubacks_per_conn, aclk_stats.mqtt.packets_waiting_puback);
 
         RRDHOST *host;
         rrd_rdlock();
@@ -1174,6 +1194,16 @@ static json_object *timestamp_to_json(const time_t *t)
 
 char *aclk_state_json(void)
 {
+
+    bool aclk_is_online = aclk_online();
+
+    struct mqtt_wss_stats aclk_stats;
+
+    if (aclk_is_online)
+        aclk_stats = aclk_statistics();
+    else
+        memset(&aclk_stats, 0, sizeof(aclk_stats));
+
     json_object *tmp, *grp, *msg = json_object_new_object();
 
     tmp = json_object_new_boolean(1);
@@ -1201,7 +1231,15 @@ char *aclk_state_json(void)
     tmp = cloud_base_url ? json_object_new_string(cloud_base_url) : NULL;
     json_object_object_add(msg, "cloud-url", tmp);
 
-    tmp = json_object_new_boolean(aclk_online());
+    char *aclk_proxy = (char *)aclk_get_proxy(NULL, true);
+    tmp = aclk_proxy ? json_object_new_string(aclk_proxy) : NULL;
+    json_object_object_add(msg, "aclk_proxy", tmp);
+
+    usec_t latency = __atomic_load_n(&publish_latency, __ATOMIC_RELAXED);
+    tmp =json_object_new_int64((int64_t) latency);
+    json_object_object_add(msg, "publish_latency_us", tmp);
+
+    tmp = json_object_new_boolean(aclk_is_online);
     json_object_object_add(msg, "online", tmp);
 
     tmp = json_object_new_string("Protobuf");
@@ -1216,13 +1254,16 @@ char *aclk_state_json(void)
     tmp = json_object_new_int(aclk_pubacks_per_conn);
     json_object_object_add(msg, "received-mqtt-pubacks", tmp);
 
+    tmp = json_object_new_int((int32_t) aclk_stats.mqtt.packets_waiting_puback);
+    json_object_object_add(msg, "pending-mqtt-pubacks", tmp);
+
     tmp = json_object_new_int(aclk_connection_counter > 0 ? (aclk_connection_counter - 1) : 0);
     json_object_object_add(msg, "reconnect-count", tmp);
 
     json_object_object_add(msg, "last-connect-time-utc", timestamp_to_json(&last_conn_time_mqtt));
     json_object_object_add(msg, "last-connect-time-puback-utc", timestamp_to_json(&last_conn_time_appl));
     json_object_object_add(msg, "last-disconnect-time-utc", timestamp_to_json(&last_disconnect_time));
-    json_object_object_add(msg, "next-connection-attempt-utc", !aclk_online() ? timestamp_to_json(&next_connection_attempt) : NULL);
+    json_object_object_add(msg, "next-connection-attempt-utc", !aclk_is_online ? timestamp_to_json(&next_connection_attempt) : NULL);
     tmp = NULL;
     if (!aclk_online() && last_backoff_value)
         tmp = json_object_new_double(last_backoff_value);
@@ -1289,7 +1330,7 @@ void add_aclk_host_labels(void) {
     rrdlabels_add(labels, "_aclk_available", "true", RRDLABEL_SRC_AUTO|RRDLABEL_SRC_ACLK);
     ACLK_PROXY_TYPE aclk_proxy;
     char *proxy_str;
-    aclk_get_proxy(&aclk_proxy);
+    aclk_get_proxy(&aclk_proxy, false);
 
     switch(aclk_proxy) {
         case PROXY_TYPE_SOCKS5:
