@@ -5,7 +5,7 @@
 #endif
 
 #include "libnetdata/libnetdata.h"
-void pulse_aclk_sent_message_acked(usec_t usec, size_t len);
+void pulse_aclk_sent_message_acked(usec_t publish_latency, size_t len);
 
 #include "common_internal.h"
 #include "mqtt_constants.h"
@@ -41,6 +41,8 @@ struct buffer_fragment {
     uint16_t packet_id;
     void (*free_fnc)(void *ptr);
     unsigned char *data;
+    // timestamp (monotonic usec) of when this MQTT packet was enqueued into the transaction buffer (set on HEAD)
+    usec_t enqueued_monotonic_ut;
     usec_t sent_monotonic_ut;
     struct buffer_fragment *next;
 };
@@ -251,6 +253,8 @@ struct mqtt_ng_client {
     size_t max_msg_size;
 };
 
+usec_t publish_latency;
+
 unsigned char pingreq[] = { MQTT_CPT_PINGREQ << 4, 0x00 };
 
 struct buffer_fragment ping_frag = {
@@ -622,6 +626,7 @@ struct mqtt_ng_client *mqtt_ng_init(struct mqtt_ng_init *settings)
     client->msg_callback = settings->msg_callback;
     spinlock_init(&client->pending_packets.spinlock);
     client->pending_packets.JudyL = NULL;
+    __atomic_store_n(&publish_latency, 0, __ATOMIC_RELEASE);
 
     return client;
 }
@@ -655,15 +660,21 @@ static void mqtt_ng_destroy_tx_alias_hash(c_rhash hash)
     c_rhash_destroy(hash);
 }
 
+static void destroy_timeout_monitor_list(struct mqtt_ng_client *client)
+{
+    spinlock_lock(&client->pending_packets.spinlock);
+    (void) JudyLFreeArray(&client->pending_packets.JudyL, PJE0);
+    spinlock_unlock(&client->pending_packets.spinlock);
+    __atomic_store_n(&client->stats.packets_waiting_puback, 0, __ATOMIC_RELAXED);
+}
+
 void mqtt_ng_destroy(struct mqtt_ng_client *client)
 {
     transaction_buffer_destroy(&client->main_buffer);
 
     mqtt_ng_destroy_tx_alias_hash(client->tx_topic_aliases.stoi_dict);
     mqtt_ng_destroy_rx_alias_hash(client->rx_aliases);
-    spinlock_lock(&client->pending_packets.spinlock);
-    (void) JudyLFreeArray(&client->pending_packets.JudyL, PJE0);
-    spinlock_unlock(&client->pending_packets.spinlock);
+    destroy_timeout_monitor_list(client);
     freez(client);
 }
 
@@ -795,26 +806,31 @@ static int optimized_add(struct header_buffer *buf, void *data, size_t data_len,
     return 0;
 }
 
-static void remove_packet_from_timeout_monitor_list(struct mqtt_ng_client *client, uint16_t packet_id)
+static void remove_packet_from_timeout_monitor_list_unsafe(struct mqtt_ng_client *client, uint16_t packet_id)
 {
-    spinlock_lock(&client->pending_packets.spinlock);
-    (void) JudyLDel(&client->pending_packets.JudyL, (Word_t) packet_id, PJE0);
-    spinlock_unlock(&client->pending_packets.spinlock);
+    int rc = JudyLDel(&client->pending_packets.JudyL, (Word_t) packet_id, PJE0);
+    // rc = 1 if the packer was deleted, so update statistics
+    if (likely(rc))
+        __atomic_fetch_sub(&client->stats.packets_waiting_puback, 1, __ATOMIC_RELAXED);
 }
 
+#define PACKET_TIMEOUT_EPOCH (1704067200L)  // Jan 1, 2024 00:00:00 UTC
+
 static void add_packet_to_timeout_monitor_list(struct mqtt_ng_client *client, uint16_t packet_id)
 {
     spinlock_lock(&client->pending_packets.spinlock);
     time_t now = now_realtime_sec();
     // Add it to the JudyL array
-    time_t *Pvalue = (time_t *) JudyLIns(&client->pending_packets.JudyL, (Word_t) packet_id, PJE0);
-    if (!Pvalue || Pvalue == PJERR) {
+    uint32_t *Pvalue = (uint32_t *) JudyLIns(&client->pending_packets.JudyL, (Word_t) packet_id, PJE0);
+    if (Pvalue == PJERR) {
         nd_log(NDLS_DAEMON, NDLP_ERR, "Error inserting packet_id (%" PRIu16 ") into JudyL array.", packet_id);
         spinlock_unlock(&client->pending_packets.spinlock);
         return;
     }
-    *Pvalue = now + PACKET_ACK_TIMEOUT_SECS;
+    *Pvalue = (uint32_t) ((now - PACKET_TIMEOUT_EPOCH) + PACKET_ACK_TIMEOUT_SECS);
     spinlock_unlock(&client->pending_packets.spinlock);
+
+    __atomic_fetch_add(&client->stats.packets_waiting_puback, 1, __ATOMIC_RELAXED);
 }
 
 #define TRY_GENERATE_MESSAGE(generator_function, ...)                                                                  \
@@ -1011,6 +1027,9 @@ int mqtt_ng_connect(struct mqtt_ng_client *client,
         buffer_purge(&client->main_buffer.hdr_buffer);
     UNLOCK_HDR_BUFFER(&client->main_buffer);
 
+    if (clean_start)
+        destroy_timeout_monitor_list(client);
+
     spinlock_lock(&client->tx_topic_aliases.spinlock);
     // according to MQTT spec topic aliases should not be persisted
     // even if clean session is true
@@ -1134,6 +1153,9 @@ int mqtt_ng_generate_publish(struct transaction_buffer *trx_buf,
     if (!qos)
         trx_buf->hdr_buffer.tail_frag->flags |= BUFFER_FRAG_GARBAGE_COLLECT_ON_SEND;
     transaction_buffer_transaction_commit(trx_buf)
+    // mark enqueue time on the HEAD fragment (after commit) so we measure from commit time
+    if (mqtt_msg)
+        mqtt_msg->enqueued_monotonic_ut = now_monotonic_usec();
     return MQTT_NG_MSGGEN_OK;
 fail_rollback:
     transaction_buffer_transaction_rollback(trx_buf, mqtt_msg);
@@ -1154,6 +1176,7 @@ static void mark_message_for_gc(struct buffer_fragment *frag)
 static int mark_packet_acked(struct mqtt_ng_client *client, uint16_t packet_id)
 {
     size_t reclaimable = 0;
+    spinlock_lock(&client->pending_packets.spinlock);
     LOCK_HDR_BUFFER(&client->main_buffer);
     struct buffer_fragment *frag = BUFFER_FIRST_FRAG(&client->main_buffer.hdr_buffer);
     while (frag) {
@@ -1161,9 +1184,14 @@ static int mark_packet_acked(struct mqtt_ng_client *client, uint16_t packet_id)
             if (!frag->sent) {
                 nd_log(NDLS_DAEMON, NDLP_ERR, "Received packet_id (%" PRIu16 ") belongs to MQTT packet which was not yet sent!", packet_id);
                 UNLOCK_HDR_BUFFER(&client->main_buffer);
+                spinlock_unlock(&client->pending_packets.spinlock);
                 return 1;
             }
-            pulse_aclk_sent_message_acked(frag->sent_monotonic_ut, frag->len);
+            // Do not reprocess this packet
+            frag->packet_id = 0;
+            usec_t latency = now_monotonic_usec() - frag->sent_monotonic_ut;
+            pulse_aclk_sent_message_acked(latency, frag->len);
+            __atomic_store_n(&publish_latency, latency, __ATOMIC_RELEASE);
             mark_message_for_gc(frag);
 
             size_t used = BUFFER_BYTES_USED(&client->main_buffer.hdr_buffer);
@@ -1171,7 +1199,8 @@ static int mark_packet_acked(struct mqtt_ng_client *client, uint16_t packet_id)
                 transaction_buffer_garbage_collect(&client->main_buffer);
 
             UNLOCK_HDR_BUFFER(&client->main_buffer);
-            remove_packet_from_timeout_monitor_list(client, packet_id);
+            remove_packet_from_timeout_monitor_list_unsafe(client, packet_id);
+            spinlock_unlock(&client->pending_packets.spinlock);
             return 0;
         }
 
@@ -1182,25 +1211,40 @@ static int mark_packet_acked(struct mqtt_ng_client *client, uint16_t packet_id)
     }
     nd_log(NDLS_DAEMON, NDLP_ERR, "Received packet_id (%" PRIu16 ") is unknown!", packet_id);
     UNLOCK_HDR_BUFFER(&client->main_buffer);
+    spinlock_unlock(&client->pending_packets.spinlock);
     return 1;
 }
 
-static void check_packet_monitor_list_for_timeouts(struct mqtt_ng_client *client)
+#define MAX_TIMED_OUT_PACKETS (1024)
+
+static bool check_packet_monitor_list_for_timeouts(struct mqtt_ng_client *client)
 {
+    uint16_t timed_out_packets[MAX_TIMED_OUT_PACKETS];
+    size_t timed_out_count = 0;
+
     spinlock_lock(&client->pending_packets.spinlock);
     bool first_then_next = true;
-    time_t *Pvalue;
+    uint32_t *Pvalue;
     Word_t packet_id = 0;
     time_t now = now_realtime_sec();
-    while ((Pvalue = (time_t *) JudyLFirstThenNext(client->pending_packets.JudyL, &packet_id, &first_then_next))) {
-        time_t expire_time = *Pvalue;
-        if (now >= expire_time) {
-            spinlock_unlock(&client->pending_packets.spinlock);
-            (void) mark_packet_acked(client, (uint16_t) packet_id);
-            spinlock_lock(&client->pending_packets.spinlock);
+
+    while ((Pvalue = (uint32_t *) JudyLFirstThenNext(client->pending_packets.JudyL, &packet_id, &first_then_next))) {
+        uint32_t expire_time_delta = *Pvalue;
+        if (now >= (PACKET_TIMEOUT_EPOCH + expire_time_delta)) {
+            if (timed_out_count < MAX_TIMED_OUT_PACKETS) {
+                timed_out_packets[timed_out_count++] = (uint16_t)packet_id;
+            } else
+                break;
         }
     }
     spinlock_unlock(&client->pending_packets.spinlock);
+
+    // Process timeouts outside the lock
+    for (size_t i = 0; i < timed_out_count; i++) {
+        mark_packet_acked(client, timed_out_packets[i]);
+    }
+
+    return (timed_out_count ==  MAX_TIMED_OUT_PACKETS);
 }
 
 #define PUBLISH_SP_SIZE 64
@@ -1237,11 +1281,6 @@ int mqtt_ng_publish(struct mqtt_ng_client *client,
     }
 
     int rc = TRY_GENERATE_MESSAGE(mqtt_ng_generate_publish, topic, topic_free, msg, msg_free, msg_len, publish_flags, packet_id, topic_id);
-    if (rc == MQTT_NG_MSGGEN_BUFFER_OOM) {
-        check_packet_monitor_list_for_timeouts(client);
-        rc = TRY_GENERATE_MESSAGE(mqtt_ng_generate_publish, topic, topic_free, msg, msg_free, msg_len, publish_flags, packet_id, topic_id);
-    }
-
     if (rc == MQTT_NG_MSGGEN_OK)
         add_packet_to_timeout_monitor_list(client, *packet_id);
     return rc;
@@ -1923,7 +1962,6 @@ static int parse_data(struct mqtt_ng_client *client)
                     }
                     parser->state = MQTT_PARSE_MQTT_PACKET_DONE;
                     ping_timeout = 0;
-                    check_packet_monitor_list_for_timeouts(client);
                     break;
                 case MQTT_CPT_DISCONNECT:
                     rc = parse_disconnect_varhdr(client);
@@ -2063,24 +2101,25 @@ int handle_incoming_traffic(struct mqtt_ng_client *client)
             client->connect_msg = NULL;
 
             if (client->client_state != MQTT_STATE_CONNECTING) {
-                nd_log(NDLS_DAEMON, NDLP_ERR, "Received unexpected CONNACK");
+                nd_log(NDLS_DAEMON, NDLP_ERR, "ACLK: Received unexpected CONNACK");
                 client->client_state = MQTT_STATE_ERROR;
                 return MQTT_NG_CLIENT_PROTOCOL_ERROR;
             }
 
             if ((prop = get_property_by_id(client->parser.properties_parser.head, MQTT_PROP_MAX_PKT_SIZE)) != NULL) {
-                nd_log(NDLS_DAEMON, NDLP_INFO, "MQTT server limits message size to %" PRIu32, prop->data.uint32);
+                nd_log(NDLS_DAEMON, NDLP_INFO, "ACLK: MQTT server limits message size to %" PRIu32, prop->data.uint32);
                 client->max_msg_size = prop->data.uint32;
             }
 
             if (client->connack_callback)
                 client->connack_callback(client->user_ctx, client->parser.mqtt_packet.connack.reason_code);
             if (!client->parser.mqtt_packet.connack.reason_code) {
-                nd_log(NDLS_DAEMON, NDLP_INFO, "MQTT Connection Accepted By Server");
+                nd_log(NDLS_DAEMON, NDLP_INFO, "ACLK: MQTT Connection Accepted By Server");
                 client->client_state = MQTT_STATE_CONNECTED;
                 break;
             }
-            client->client_state = MQTT_STATE_ERROR;            return MQTT_NG_CLIENT_SERVER_RETURNED_ERROR;
+            client->client_state = MQTT_STATE_ERROR;
+            return MQTT_NG_CLIENT_SERVER_RETURNED_ERROR;
 
         case MQTT_CPT_PUBACK:
             worker_is_busy(WORKER_ACLK_CPT_PUBACK);
@@ -2093,7 +2132,8 @@ int handle_incoming_traffic(struct mqtt_ng_client *client)
 
         case MQTT_CPT_PINGRESP:
             worker_is_busy(WORKER_ACLK_CPT_PINGRESP);
-            pulse_aclk_sent_message_acked(ping_frag.sent_monotonic_ut, ping_frag.len);
+            usec_t latency = now_monotonic_usec() - ping_frag.sent_monotonic_ut;
+            pulse_aclk_sent_message_acked(latency, ping_frag.len);
             break;
 
         case MQTT_CPT_SUBACK:
@@ -2163,6 +2203,8 @@ int handle_incoming_traffic(struct mqtt_ng_client *client)
     return rc;
 }
 
+#define PACKET_TIMEOUT_REPEAT_CHECK (60)
+
 int mqtt_ng_sync(struct mqtt_ng_client *client)
 {
     if (client->client_state == MQTT_STATE_RAW || client->client_state == MQTT_STATE_DISCONNECTED)
@@ -2171,6 +2213,15 @@ int mqtt_ng_sync(struct mqtt_ng_client *client)
     if (client->client_state == MQTT_STATE_ERROR)
         return 1;
 
+    // Check for packet timeouts and cleanup
+    static time_t last_maintenance = 0;
+    if (now_realtime_sec() - last_maintenance >= PACKET_TIMEOUT_REPEAT_CHECK) {
+        // if check packet returns true then we did max cleanup, possibly there are more packets to cleanup
+        // so do not update last_maintenance thus forcing check again
+        if (likely(!check_packet_monitor_list_for_timeouts(client)))
+            last_maintenance = now_realtime_sec();
+    }
+
     worker_is_busy(WORKER_ACLK_TRY_SEND_ALL);
 
     LOCK_HDR_BUFFER(&client->main_buffer);
@@ -2218,20 +2269,73 @@ void mqtt_ng_get_stats(struct mqtt_ng_client *client, struct mqtt_ng_stats *stat
 
     stats->tx_bytes_queued = 0;
     stats->tx_buffer_reclaimable = 0;
+    stats->max_puback_wait_us = 0;
+    stats->max_send_queue_wait_us = 0;
+    stats->max_unsent_wait_us = 0;
+    stats->max_partial_wait_us = 0;
 
+    // First pass: compute buffer usage/queued bytes and max send-queue wait time (unsent messages)
     LOCK_HDR_BUFFER(&client->main_buffer);
     stats->tx_buffer_used = BUFFER_BYTES_USED(&client->main_buffer.hdr_buffer);
     stats->tx_buffer_free = BUFFER_BYTES_AVAILABLE(&client->main_buffer.hdr_buffer);
     stats->tx_buffer_size = client->main_buffer.hdr_buffer.size;
     struct buffer_fragment *frag = BUFFER_FIRST_FRAG(&client->main_buffer.hdr_buffer);
+    usec_t now_ut = now_monotonic_usec();
     while (frag) {
         stats->tx_bytes_queued += frag->len - frag->sent;
         if (frag_is_marked_for_gc(frag))
             stats->tx_buffer_reclaimable += FRAG_SIZE_IN_BUFFER(frag);
 
+        // For HEAD fragments that are not fully sent yet (unsent or partially sent),
+        // track max send-queue wait time. Prefer the enqueue timestamp; if missing, fall back to first-send.
+        if ((frag->flags & BUFFER_FRAG_MQTT_PACKET_HEAD) && frag->sent < frag->len) {
+            usec_t base = frag->enqueued_monotonic_ut ? frag->enqueued_monotonic_ut : frag->sent_monotonic_ut;
+            if (base) {
+                usec_t waited = now_ut - base;
+                uint64_t w = (uint64_t)waited;
+                if (frag->sent == 0) {
+                    if (w > stats->max_unsent_wait_us) stats->max_unsent_wait_us = w;
+                } else {
+                    if (w > stats->max_partial_wait_us) stats->max_partial_wait_us = w;
+                }
+                if (w > stats->max_send_queue_wait_us) stats->max_send_queue_wait_us = w;
+            } else {
+                // Throttled debug if enqueue time is missing on an unsent HEAD
+                if (frag->sent == 0) {
+                    static time_t last_warn = 0;
+                    time_t now_s = now_monotonic_sec();
+                    if (now_s - last_warn > 60) {
+                        nd_log(NDLS_DAEMON, NDLP_DEBUG, "ACLK: Missing enqueue timestamp on unsent MQTT packet head");
+                        last_warn = now_s;
+                    }
+                }
+            }
+        }
+
         frag = frag->next;
     }
     UNLOCK_HDR_BUFFER(&client->main_buffer);
+
+    // Second pass: compute max PUBACK wait time by correlating HEAD fragments with pending packet IDs
+    spinlock_lock(&client->pending_packets.spinlock);
+    LOCK_HDR_BUFFER(&client->main_buffer);
+    frag = BUFFER_FIRST_FRAG(&client->main_buffer.hdr_buffer);
+    while (frag) {
+        if ((frag->flags & BUFFER_FRAG_MQTT_PACKET_HEAD) && frag->packet_id) {
+            Pvoid_t *Pvalue = JudyLGet(client->pending_packets.JudyL, (Word_t)frag->packet_id, PJE0);
+            if (Pvalue) {
+                // message is still pending PUBACK
+                if (frag->sent_monotonic_ut) {
+                    usec_t waited = now_ut - frag->sent_monotonic_ut;
+                    if ((uint64_t)waited > stats->max_puback_wait_us)
+                        stats->max_puback_wait_us = (uint64_t)waited;
+                }
+            }
+        }
+        frag = frag->next;
+    }
+    UNLOCK_HDR_BUFFER(&client->main_buffer);
+    spinlock_unlock(&client->pending_packets.spinlock);
 }
 
 int mqtt_ng_set_topic_alias(struct mqtt_ng_client *client, const char *topic)
