@@ -611,15 +611,30 @@ void call_the_helper(pid_t pid, const char *cgroup) {
         nd_log(NDLS_COLLECTORS, NDLP_ERR, "cannot execute cgroup-network helper script: %s", command);
 }
 
+int ishex(char c) {
+    return (c >= '0' && c <= '9') ||
+           (c >= 'a' && c <= 'f') ||
+           (c >= 'A' && c <= 'F');
+}
+
+int is_valid_hex_escape(const char *arg) {
+    fatal_assert(arg);
+
+    return (arg[0] == '\\') &&
+           (arg[1] == 'x') &&
+           ishex(arg[2]) &&
+           ishex(arg[3]);
+}
+
 int is_valid_path_symbol(char c) {
     switch(c) {
         case '/':   // path separators
-        case '\\':  // needed for virsh domains \x2d1\x2dname
         case ' ':   // space
         case '-':   // hyphen
         case '_':   // underscore
         case '.':   // dot
         case ',':   // comma
+        case '@':   // systemd unit template specifier (/sys/fs/cgroup/machines.slice/systemd-nspawn@NAME.service)
             return 1;
 
         default:
@@ -635,20 +650,20 @@ int is_valid_path_symbol(char c) {
 int verify_path(const char *path) {
     struct stat sb;
 
-    char c;
+    fatal_assert(path);
+
     const char *s = path;
-    while((c = *s++)) {
-        if(!( isalnum(c) || is_valid_path_symbol(c) )) {
+    while(*s != '\0') {
+        if (isalnum(*s) || is_valid_path_symbol(*s))
+            s += 1;
+        else if (*s == '\\' && is_valid_hex_escape(s))
+            s += 4;
+        else {
             nd_log(NDLS_COLLECTORS, NDLP_ERR, "invalid character in path '%s'", path);
             return -1;
         }
     }
 
-    if(strstr(path, "\\") && !strstr(path, "\\x")) {
-        nd_log(NDLS_COLLECTORS, NDLP_ERR, "invalid escape sequence in path '%s'", path);
-        return 1;
-    }
-
     if(strstr(path, "/../")) {
         nd_log(NDLS_COLLECTORS, NDLP_ERR, "invalid parent path sequence detected in '%s'", path);
         return 1;
