@@ -153,36 +153,36 @@ int do_getifaddrs(int update_every, usec_t dt) {
     static SIMPLE_PATTERN *excluded_interfaces = NULL, *physical_interfaces = NULL;
 
     if (unlikely(enable_new_interfaces == -1)) {
-        enable_new_interfaces = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS,
+        enable_new_interfaces = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS,
                                                               "enable new interfaces detected at runtime",
                                                               CONFIG_BOOLEAN_AUTO);
 
-        do_bandwidth_net  = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "total bandwidth for physical interfaces",
+        do_bandwidth_net  = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "total bandwidth for physical interfaces",
                                                        CONFIG_BOOLEAN_AUTO);
-        do_packets_net    = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "total packets for physical interfaces",
+        do_packets_net    = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "total packets for physical interfaces",
                                                        CONFIG_BOOLEAN_AUTO);
-        do_bandwidth_ipv4 = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "total bandwidth for ipv4 interfaces",
+        do_bandwidth_ipv4 = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "total bandwidth for ipv4 interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_bandwidth_ipv6 = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "total bandwidth for ipv6 interfaces",
+        do_bandwidth_ipv6 = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "total bandwidth for ipv6 interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_bandwidth      = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "bandwidth for all interfaces",
+        do_bandwidth      = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "bandwidth for all interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_packets        = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "packets for all interfaces",
+        do_packets        = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "packets for all interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_errors         = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "errors for all interfaces",
+        do_errors         = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "errors for all interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_drops          = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "drops for all interfaces",
+        do_drops          = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "drops for all interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
-        do_events         = config_get_boolean_ondemand(CONFIG_SECTION_GETIFADDRS, "collisions for all interfaces",
+        do_events         = inicfg_get_boolean_ondemand(&netdata_config, CONFIG_SECTION_GETIFADDRS, "collisions for all interfaces",
                                                         CONFIG_BOOLEAN_AUTO);
 
         excluded_interfaces = simple_pattern_create(
-            config_get(CONFIG_SECTION_GETIFADDRS, "disable by default interfaces matching", DEFAULT_EXCLUDED_INTERFACES),
+            inicfg_get(&netdata_config, CONFIG_SECTION_GETIFADDRS, "disable by default interfaces matching", DEFAULT_EXCLUDED_INTERFACES),
             NULL,
             SIMPLE_PATTERN_EXACT,
             true);
         physical_interfaces = simple_pattern_create(
-            config_get(CONFIG_SECTION_GETIFADDRS, "set physical interfaces for system.net", DEFAULT_PHYSICAL_INTERFACES),
+            inicfg_get(&netdata_config, CONFIG_SECTION_GETIFADDRS, "set physical interfaces for system.net", DEFAULT_PHYSICAL_INTERFACES),
             NULL,
             SIMPLE_PATTERN_EXACT,
             true);
@@ -406,16 +406,16 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->enabled = !simple_pattern_matches(excluded_interfaces, ifa->ifa_name);
 
                     snprintfz(var_name, 4096, "%s:%s", CONFIG_SECTION_GETIFADDRS, ifa->ifa_name);
-                    ifm->enabled = config_get_boolean_ondemand(var_name, "enabled", ifm->enabled);
+                    ifm->enabled = inicfg_get_boolean_ondemand(&netdata_config, var_name, "enabled", ifm->enabled);
 
                     if (unlikely(ifm->enabled == CONFIG_BOOLEAN_NO))
                         continue;
 
-                    ifm->do_bandwidth = config_get_boolean_ondemand(var_name, "bandwidth", do_bandwidth);
-                    ifm->do_packets   = config_get_boolean_ondemand(var_name, "packets",   do_packets);
-                    ifm->do_errors    = config_get_boolean_ondemand(var_name, "errors",    do_errors);
-                    ifm->do_drops     = config_get_boolean_ondemand(var_name, "drops",     do_drops);
-                    ifm->do_events    = config_get_boolean_ondemand(var_name, "events",    do_events);
+                    ifm->do_bandwidth = inicfg_get_boolean_ondemand(&netdata_config, var_name, "bandwidth", do_bandwidth);
+                    ifm->do_packets   = inicfg_get_boolean_ondemand(&netdata_config, var_name, "packets",   do_packets);
+                    ifm->do_errors    = inicfg_get_boolean_ondemand(&netdata_config, var_name, "errors",    do_errors);
+                    ifm->do_drops     = inicfg_get_boolean_ondemand(&netdata_config, var_name, "drops",     do_drops);
+                    ifm->do_events    = inicfg_get_boolean_ondemand(&netdata_config, var_name, "events",    do_events);
                 }
 
                 if (unlikely(!ifm->enabled))
@@ -426,7 +426,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->st_bandwidth = rrdset_create_localhost("net",
                                                                     ifa->ifa_name,
                                                                     NULL,
-                                                                    ifa->ifa_name,
+                                                                    "traffik",
                                                                     "net.net",
                                                                     "Bandwidth",
                                                                     "kilobits/s",
@@ -439,6 +439,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
 
                         ifm->rd_bandwidth_in  = rrddim_add(ifm->st_bandwidth, "received", NULL,  8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
                         ifm->rd_bandwidth_out = rrddim_add(ifm->st_bandwidth, "sent",     NULL, -8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
+                        rrdlabels_add(ifm->st_bandwidth->rrdlabels, "device", ifa->ifa_name, RRDLABEL_SRC_AUTO);
                     }
 
                     rrddim_set_by_pointer(ifm->st_bandwidth, ifm->rd_bandwidth_in,  IFA_DATA(ibytes));
@@ -451,7 +452,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->st_packets = rrdset_create_localhost("net_packets",
                                                                   ifa->ifa_name,
                                                                   NULL,
-                                                                  ifa->ifa_name,
+                                                                  "packets",
                                                                   "net.packets",
                                                                   "Packets",
                                                                   "packets/s",
@@ -470,6 +471,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                                                            RRD_ALGORITHM_INCREMENTAL);
                         ifm->rd_packets_m_out = rrddim_add(ifm->st_packets, "multicast_sent",     NULL, -1, 1,
                                                            RRD_ALGORITHM_INCREMENTAL);
+                        rrdlabels_add(ifm->st_packets->rrdlabels, "device", ifa->ifa_name, RRDLABEL_SRC_AUTO);
                     }
 
                     rrddim_set_by_pointer(ifm->st_packets, ifm->rd_packets_in,    IFA_DATA(ipackets));
@@ -484,7 +486,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->st_errors = rrdset_create_localhost("net_errors",
                                                                  ifa->ifa_name,
                                                                  NULL,
-                                                                 ifa->ifa_name,
+                                                                 "errors",
                                                                  "net.errors",
                                                                  "Interface Errors",
                                                                  "errors/s",
@@ -497,6 +499,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
 
                         ifm->rd_errors_in  = rrddim_add(ifm->st_errors, "inbound",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                         ifm->rd_errors_out = rrddim_add(ifm->st_errors, "outbound", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
+                        rrdlabels_add(ifm->st_errors->rrdlabels, "device", ifa->ifa_name, RRDLABEL_SRC_AUTO);
                     }
 
                     rrddim_set_by_pointer(ifm->st_errors, ifm->rd_errors_in,  IFA_DATA(ierrors));
@@ -509,7 +512,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->st_drops = rrdset_create_localhost("net_drops",
                                                                 ifa->ifa_name,
                                                                 NULL,
-                                                                ifa->ifa_name,
+                                                                "drops",
                                                                 "net.drops",
                                                                 "Interface Drops",
                                                                 "drops/s",
@@ -524,6 +527,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
 #if __FreeBSD__ >= 11
                         ifm->rd_drops_out = rrddim_add(ifm->st_drops, "outbound", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
 #endif
+                        rrdlabels_add(ifm->st_drops->rrdlabels, "device", ifa->ifa_name, RRDLABEL_SRC_AUTO);
                     }
 
                     rrddim_set_by_pointer(ifm->st_drops, ifm->rd_drops_in,  IFA_DATA(iqdrops));
@@ -538,7 +542,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
                         ifm->st_events = rrdset_create_localhost("net_events",
                                                                  ifa->ifa_name,
                                                                  NULL,
-                                                                 ifa->ifa_name,
+                                                                 "errors",
                                                                  "net.events",
                                                                  "Network Interface Events",
                                                                  "events/s",
@@ -551,6 +555,7 @@ int do_getifaddrs(int update_every, usec_t dt) {
 
                         ifm->rd_events_coll = rrddim_add(ifm->st_events, "collisions", NULL, -1, 1,
                                                          RRD_ALGORITHM_INCREMENTAL);
+                        rrdlabels_add(ifm->st_events->rrdlabels, "device", ifa->ifa_name, RRDLABEL_SRC_AUTO);
                     }
 
                     rrddim_set_by_pointer(ifm->st_events, ifm->rd_events_coll, IFA_DATA(collisions));
