@@ -6,7 +6,7 @@
 #define PLUGIN_PROC_MODULE_NETDEV_NAME "/proc/net/dev"
 #define CONFIG_SECTION_PLUGIN_PROC_NETDEV "plugin:" PLUGIN_PROC_CONFIG_NAME ":" PLUGIN_PROC_MODULE_NETDEV_NAME
 
-#define RRDFUNCTIONS_NETDEV_HELP "View network interface statistics"
+#define RRDFUNCTIONS_NETDEV_HELP "Shows real-time network interface performance including traffic rates, packet counts, drops, and link status."
 
 #define STATE_LENGTH_MAX 32
 
@@ -379,7 +379,15 @@ static void netdev_free(struct netdev *d) {
     freez((void *)d);
 }
 
-static netdata_mutex_t netdev_mutex = NETDATA_MUTEX_INITIALIZER;
+static netdata_mutex_t netdev_mutex;
+
+static void __attribute__((constructor)) init_mutex(void) {
+    netdata_mutex_init(&netdev_mutex);
+}
+
+static void __attribute__((destructor)) destroy_mutex(void) {
+    netdata_mutex_destroy(&netdev_mutex);
+}
 
 // ----------------------------------------------------------------------------
 
