@@ -238,6 +238,44 @@ static const char *drive_type_to_str(UINT type)
     }
 }
 
+static inline LONGLONG convertToBytes(LONGLONG value, double factor) {
+    double dvalue = value;
+    dvalue /= (factor);
+
+    return (LONGLONG) dvalue*100;
+}
+
+static inline void netdata_set_hd_usage(PERF_DATA_BLOCK *pDataBlock,
+                                        PERF_OBJECT_TYPE *pObjectType,
+                                        PERF_INSTANCE_DEFINITION *pi,
+                                        struct logical_disk *d)
+{
+    ULARGE_INTEGER totalNumberOfBytes;
+    ULARGE_INTEGER totalNumberOfFreeBytes;
+
+// https://learn.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation?tabs=registry
+#define MAX_DRIVE_LENGTH 255
+    char path[MAX_DRIVE_LENGTH + 1];
+    snprintfz(path, MAX_DRIVE_LENGTH, "%s\\", windows_shared_buffer);
+
+    // Description of incompatibilities present in both methods we are using
+    // https://devblogs.microsoft.com/oldnewthing/20071101-00/?p=24613
+    // We are using the variable that should not be affected by qyota ()
+    if ((GetDriveTypeA(path) != DRIVE_FIXED) || !GetDiskFreeSpaceExA(path,
+                                                                     NULL,
+                                                                     &totalNumberOfBytes,
+                                                                     &totalNumberOfFreeBytes)) {
+        perflibGetInstanceCounter(pDataBlock, pObjectType, pi, &d->percentDiskFree);
+
+        d->percentDiskFree.current.Data = convertToBytes(d->percentDiskFree.current.Data, 1024);
+        d->percentDiskFree.current.Time = convertToBytes(d->percentDiskFree.current.Time, 1024);
+        return;
+    }
+
+    d->percentDiskFree.current.Data = convertToBytes(totalNumberOfFreeBytes.QuadPart, 1024 * 1024 * 1024);
+    d->percentDiskFree.current.Time = convertToBytes(totalNumberOfBytes.QuadPart, 1024 * 1024 * 1024);
+}
+
 static bool do_logical_disk(PERF_DATA_BLOCK *pDataBlock, int update_every, usec_t now_ut)
 {
     DICTIONARY *dict = logicalDisks;
@@ -266,7 +304,7 @@ static bool do_logical_disk(PERF_DATA_BLOCK *pDataBlock, int update_every, usec_
             d->collected_metadata = true;
         }
 
-        perflibGetInstanceCounter(pDataBlock, pObjectType, pi, &d->percentDiskFree);
+        netdata_set_hd_usage(pDataBlock, pObjectType, pi, d);
         // perflibGetInstanceCounter(pDataBlock, pObjectType, pi, &d->freeMegabytes);
 
         if (!d->st_disk_space) {
@@ -300,8 +338,8 @@ static bool do_logical_disk(PERF_DATA_BLOCK *pDataBlock, int update_every, usec_
                 rrdlabels_add(d->st_disk_space->rrdlabels, "serial_number", buf, RRDLABEL_SRC_AUTO);
             }
 
-            d->rd_disk_space_free = rrddim_add(d->st_disk_space, "avail", NULL, 1, 1024, RRD_ALGORITHM_ABSOLUTE);
-            d->rd_disk_space_used = rrddim_add(d->st_disk_space, "used", NULL, 1, 1024, RRD_ALGORITHM_ABSOLUTE);
+            d->rd_disk_space_free = rrddim_add(d->st_disk_space, "avail", NULL, 1, 100, RRD_ALGORITHM_ABSOLUTE);
+            d->rd_disk_space_used = rrddim_add(d->st_disk_space, "used", NULL, 1, 100, RRD_ALGORITHM_ABSOLUTE);
         }
 
         // percentDiskFree has the free space in Data and the size of the disk in Time, in MiB.
