@@ -11,13 +11,21 @@
 // 2^24
 #define WINDOWS_MAX_KERNEL_OBJECT 16777216
 
+#ifndef MEGA_FACTOR
+#define MEGA_FACTOR (1048576)
+#endif
+
 void *win_plugin_main(void *ptr);
 
 extern char windows_shared_buffer[8192];
 
+// Windows API
 int do_GetSystemUptime(int update_every, usec_t dt);
 int do_GetSystemRAM(int update_every, usec_t dt);
 int do_GetSystemCPU(int update_every, usec_t dt);
+int do_GetPowerSupply(int update_every, usec_t dt);
+
+// Perflib
 int do_PerflibStorage(int update_every, usec_t dt);
 int do_PerflibNetwork(int update_every, usec_t dt);
 int do_PerflibProcesses(int update_every, usec_t dt);
@@ -33,6 +41,9 @@ int do_PerflibADCS(int update_every, usec_t dt);
 int do_PerflibADFS(int update_every, usec_t dt);
 int do_PerflibHyperV(int update_every, usec_t dt);
 int do_PerflibServices(int update_every, usec_t dt);
+int do_PerflibExchange(int update_every, usec_t dt __maybe_unused);
+int do_PerflibNUMA(int update_every, usec_t dt __maybe_unused);
+int do_PerflibASP(int update_every, usec_t dt __maybe_unused);
 
 enum PERFLIB_PRIO {
     PRIO_WEBSITE_IIS_REQUESTS_RATE = 21000, // PRIO selected, because APPS is using 20YYY
@@ -60,6 +71,20 @@ enum PERFLIB_PRIO {
     PRIO_IIS_APP_POOL_TOTAL_WORKER_PROCESS_STARTUP_FAILURES,
     PRIO_IIS_APP_POOL_TOTAL_UPTIME,
 
+    PRIO_W3SVC_W3WP_ACTIVE_THREADS,
+    PRIO_W3SVC_W3WP_REQUESTS_TOTAL,
+    PRIO_W3SVC_W3WP_REQUESTS_ACTIVE,
+    PRIO_W3SVC_W3WP_FILE_CACHE_MEM_USAGE,
+    PRIO_W3SVC_W3WP_FILE_CACHE_TOTAL,
+    PRIO_W3SVC_W3WP_FILE_FLUSHED_TOTAL,
+    PRIO_W3SVC_W3WP_URI_FLUSHED_TOTAL,
+    PRIO_W3SVC_W3WP_URI_CACHED_TOTAL,
+    PRIO_W3SVC_W3WP_METADATA_CACHED,
+    PRIO_W3SVC_W3WP_METADATA_FLUSHED,
+    PRIO_W3SVC_W3WP_OUTPUT_CACHE_ACTIVE_FLUSH,
+    PRIO_W3SVC_W3WP_OUTPUT_CACHE_MEMORY_USAGE,
+    PRIO_W3SVC_W3WP_OUTPUT_CACHE_FLUSHED_TOTAL,
+
     PRIO_MSSQL_USER_CONNECTIONS,
 
     PRIO_MSSQL_DATABASE_TRANSACTIONS,
@@ -98,6 +123,12 @@ enum PERFLIB_PRIO {
     PRIO_MSSQL_LOCKS_WAIT,
     PRIO_MSSQL_LOCKS_DEADLOCK_PER_RESOURCE,
 
+    PRIO_MSSQL_WAITS_TOTAL,
+    PRIO_MSSQL_RESOURCE_WAIT,
+    PRIO_MSSQL_SIGNAL_WAIT,
+    PRIO_MSSQL_MAX_WAIT_TIME,
+    PRIO_MSSQL_WAITING_COUNT,
+
     PRIO_MSSQL_SQL_ERRORS,
 
     PRIO_NETFRAMEWORK_CLR_EXCEPTION_THROWN,
@@ -200,7 +231,72 @@ enum PERFLIB_PRIO {
     PRIO_ADFS_SSO_AUTH,
     PRIO_ADFS_TRUST_TOKEN_SUCCESS,
     PRIO_ADFS_USER_PASS_AUTH,
-    PRIO_ADFS_WSFED_TOKEN_REQUESTS
+    PRIO_ADFS_WSFED_TOKEN_REQUESTS,
+
+    PRIO_EXCHANGE_QUEUE_ACTIVE_MAILBOX_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_EXTERNAL_ACTIVE_REMOTE_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_EXTERNAL_LARGEST_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_INTERNAL_ACTIVE_REMOTE_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_INTERNAL_LARGEST_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_RETRY_MAILBOX_DELIVERY,
+    PRIO_EXCHANGE_QUEUE_UNREACHABLE,
+    PRIO_EXCHANGE_QUEUE_POISON,
+    PRIO_EXCHANGE_ACTIVE_SYNC_PING_CMDS_PENDING,
+    PRIO_EXCHANGE_ACTIVE_SYNC_REQUESTS,
+    PRIO_EXCHANGE_ACTIVE_SYNC_CMDS,
+    PRIO_EXCHANGE_RPC_ACTIVE_USERS_COUNT,
+    PRIO_EXCHANGE_RPC_AVG_LATENCY,
+    PRIO_EXCHANGE_RPC_CONNECTION_COUNT,
+    PRIO_EXCHANGE_RPC_OPERATIONS_TOTAL,
+    PRIO_EXCHANGE_RPC_REQUESTS,
+    PRIO_EXCHANGE_RPC_USER_COUNT,
+    PRIO_EXCHANGE_OWA_REQUESTS_TOTAL,
+    PRIO_EXCHANGE_OWA_UNIQUE_USERS,
+    PRIO_EXCHANGE_WORKlOAD_ACTIVE_TASKS,
+    PRIO_EXCHANGE_WORKlOAD_COMPLETE_TASKS,
+    PRIO_EXCHANGE_WORKlOAD_QUEUE_TASKS,
+    PRIO_EXCHANGE_WORKlOAD_YIELDED_TASKS,
+    PRIO_EXCHANGE_WORKlOAD_ACTIVITY,
+    PRIO_EXCHANGE_PROXY_AVG_AUTH_LATENCY,
+    PRIO_EXCHANGE_PROXY_AVG_CAS_PROCESSING_LATENCY,
+    PRIO_EXCHANGE_PROXY_MAILBOX_PROXY_FAILURE,
+    PRIO_EXCHANGE_PROXY_SERVER_LOCATIOR_AVG_LATENCY,
+    PRIO_EXCHANGE_PROXY_OUTSTANDING_PROXY_REQUESTS,
+    PRIO_EXCHANGE_PROXY_PROXY_REQUESTS_TOTAL,
+    PRIO_EXCHANGE_AUTO_DISCOVER_REQUESTS,
+    PRIO_EXCHANGE_AUTO_AVAILABILITY_SERVICES,
+
+    PRIO_ASPNET_APPLICATION_RESTART,
+    PRIO_ASPNET_WORKER_PROCESS_RESTART,
+    PRIO_ASPNET_APP_ANONYMOUS_REQUEST,
+    PRIO_ASPNET_APP_COMPILATION_TOTALS,
+    PRIO_ASPNET_ERRORS_DURING_PREPROCESSING,
+    PRIO_ASPNET_ERRORS_DURING_COMPILATION,
+    PRIO_ASPNET_ERRORS_DURING_EXECUTION,
+    PRIO_ASPNET_ERRORS_UNHANDLED_EXECUTION_PER_SEC,
+    PRIO_ASPNET_REQUESTS_BYTES_TOTAL,
+    PRIO_ASPNET_REQUESTS_EXECUTING,
+    PRIO_ASPNET_REQUESTS_FAILED,
+    PRIO_ASPNET_REQUESTS_NOT_FOUND,
+    PRIO_ASPNET_REQUESTS_IN_APPLICATION_QUEUE,
+    PRIO_ASPNET_REQUESTS_NOT_AUTHORIZED,
+    PRIO_ASPNET_REQUESTS_TIMEOUT,
+    PRIO_ASPNET_REQUESTS_SUCCEEDED,
+    PRIO_ASPNET_SESSIONS_ACTIVE,
+    PRIO_ASPNET_SESSIONS_ABANDONED,
+    PRIO_ASPNET_SESSIONS_TIMEDOUT,
+    PRIO_ASPNET_TRANSACTIONS_ABORTED,
+    PRIO_ASPNET_TRANSACTIONS_COMMITED,
+    PRIO_ASPNET_TRANSACTIONS_PENDING,
+    PRIO_ASPNET_TRANSACTIONS_PER_SEC,
+    PRIO_ASPNET_EVENTS_RAISED_PER_SEC,
+    PRIO_ASPNET_ERROR_EVENTS_RAISED_PER_SEC,
+    PRIO_ASPNET_AUDIT_SUCCESS_EVENTS_RAISED,
+    PRIO_ASPNET_AUDIT_FAILURES_EVENTS_RAISED,
+    PRIO_ASPNET_MEMBERSHIP_AUTHENTICATION_SUCCESS,
+    PRIO_ASPNET_MEMBERSHIP_AUTHENTICATION_FAILURE,
+    PRIO_ASPNET_FORM_AUTHENTICATION_SUCCESS,
+    PRIO_ASPNET_FORM_AUTHENTICATION_FAILURE
 };
 
 #endif //NETDATA_WINDOWS_PLUGIN_H
