@@ -86,8 +86,60 @@ enum event_loop_job {
     UV_EVENT_SCHEDULE_CMD,
 };
 
+#define MAX_ACTIVE_WORKERS (256)
+
+typedef struct worker_data {
+    uv_work_t request;
+    void *config;
+    void *pending_alert_list;
+    void *pending_ctx_cleanup_list;
+    void *pending_uuid_deletion;
+    void *pending_sql_statement;
+    union {
+        void *payload;
+        void *work_buffer;
+    };
+    bool allocated;
+} worker_data_t;
+
+typedef struct {
+    worker_data_t workers[MAX_ACTIVE_WORKERS];  // Preallocated worker data pool
+    int free_stack[MAX_ACTIVE_WORKERS];  // Stack of available worker data indices
+    int top;  // Stack pointer
+} WorkerPool;
+
+typedef struct {
+    uint8_t opcode;
+    uint8_t padding[sizeof(void *) - sizeof(uint8_t)]; // Padding to align the union
+    union {
+        void *param[2];
+        char data[sizeof(void *) * 2];
+    };
+} cmd_data_t;
+
+typedef struct {
+    cmd_data_t *buffer;
+    int size;
+    int head;
+    int tail;
+    int count;
+
+    uv_mutex_t lock;
+    uv_cond_t not_full;
+} CmdPool;
+
+
 void register_libuv_worker_jobs();
-int create_uv_thread(uv_thread_t *thread, uv_thread_cb thread_func, void *arg, int *retries);
 void libuv_close_callback(uv_handle_t *handle, void *data __maybe_unused);
 
+void init_worker_pool(WorkerPool *pool);
+worker_data_t *get_worker(WorkerPool *pool);
+void return_worker(WorkerPool *pool, worker_data_t *worker);
+
+void init_cmd_pool(CmdPool *pool, int size);
+bool push_cmd(CmdPool *pool, const cmd_data_t *cmd, bool wait_on_full);
+bool pop_cmd(CmdPool *pool, cmd_data_t *out_cmd);
+void release_cmd_pool(CmdPool *pool);
+int test_cmd_pool_fifo();
+
 #endif //NETDATA_EVENT_LOOP_H
