@@ -217,6 +217,7 @@ int uuid_unittest(void);
 int progress_unittest(void);
 int dyncfg_unittest(void);
 int eval_unittest(void);
+int duration_unittest(void);
 bool netdata_random_session_id_generate(void);
 
 #ifdef OS_WINDOWS
@@ -404,9 +405,11 @@ int netdata_main(int argc, char **argv) {
                             if (uuid_unittest()) return 1;
                             if (dyncfg_unittest()) return 1;
                             if (eval_unittest()) return 1;
+                            if (duration_unittest()) return 1;
                             if (unittest_waiting_queue()) return 1;
                             if (uuidmap_unittest()) return 1;
                             if (stacktrace_unittest()) return 1;
+                            if (test_cmd_pool_fifo()) return 1;
 #ifdef OS_WINDOWS
                             if (perflibnamestest_main()) return 1;
 #endif
@@ -457,6 +460,10 @@ int netdata_main(int argc, char **argv) {
                             unittest_running = true;
                             return buffer_unittest();
                         }
+                        else if(strcmp(optarg, "test_cmd_pool_fifo") == 0) {
+                            unittest_running = true;
+                            return test_cmd_pool_fifo();
+                        }
                         else if(strcmp(optarg, "uuidtest") == 0) {
                             unittest_running = true;
                             return uuid_unittest();
@@ -511,6 +518,10 @@ int netdata_main(int argc, char **argv) {
                             unittest_running = true;
                             return eval_unittest();
                         }
+                        else if(strcmp(optarg, "durationtest") == 0) {
+                            unittest_running = true;
+                            return duration_unittest();
+                        }
                         else if(strcmp(optarg, "dyncfgtest") == 0) {
                             unittest_running = true;
                             if(unittest_prepare_rrd(&user))
@@ -1109,17 +1120,6 @@ int netdata_main(int argc, char **argv) {
         else // collect data but do not send it (needed in /api/v1/info)
             delta_startup_time("anonymous analytics (disabled)");
 
-        analytics_statistic_t start_statistic = {"START", "-", "-"};
-        analytics_statistic_send(&start_statistic);
-        if (daemon_status_file_has_last_crashed(NULL)) {
-            analytics_statistic_t crash_statistic = {"CRASH", "-", "-"};
-            analytics_statistic_send(&crash_statistic);
-        }
-        if (daemon_status_file_was_incomplete_shutdown()) {
-            analytics_statistic_t incomplete_shutdown_statistic = {"INCOMPLETE_SHUTDOWN", "-", "-"};
-            analytics_statistic_send(&incomplete_shutdown_statistic);
-        }
-
         // check if ANALYTICS needs to start
         for (i = 0; static_threads[i].name != NULL; i++) {
             if (!strncmp(static_threads[i].name, "ANALYTICS", 9)) {
