@@ -296,7 +296,16 @@ void journalfile_v2_data_unmount_cleanup(time_t now_s) {
         if(uv_rwlock_tryrdlock(&ctx->datafiles.rwlock) != 0)
             continue;
 
-        for (datafile = ctx->datafiles.first; datafile; datafile = datafile->next) {
+        bool first_then_next = true;
+        Pvoid_t *Pvalue =  NULL;
+        Word_t Index = 0;
+
+        while((Pvalue = JudyLFirstThenNext(ctx->datafiles.JudyL, &Index, &first_then_next))) {
+
+            datafile = *Pvalue;
+            if (!datafile)
+                continue;
+
             struct rrdengine_journalfile *journalfile = datafile->journalfile;
 
             if(!spinlock_trylock(&journalfile->data_spinlock))
@@ -477,14 +486,10 @@ static int close_uv_file(struct rrdengine_datafile *datafile, uv_file file)
     int ret;
     char path[RRDENG_PATH_MAX];
 
-    uv_fs_t req;
-    ret = uv_fs_close(NULL, &req, file, NULL);
-    if (ret < 0) {
-        journalfile_v1_generate_path(datafile, path, sizeof(path));
-        netdata_log_error("DBENGINE: uv_fs_close(\"%s\"): %s", path, uv_strerror(ret));
-        ctx_fs_error(datafile_ctx(datafile));
-    }
-    uv_fs_req_cleanup(&req);
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
+    journalfile_v1_generate_path(datafile, path, sizeof(path));
+
+    CLOSE_FILE(ctx, path, file, ret);
     return ret;
 }
 
@@ -1524,7 +1529,6 @@ bool journalfile_migrate_to_v2_callback(Word_t section, unsigned datafile_fileno
 int journalfile_load(struct rrdengine_instance *ctx, struct rrdengine_journalfile *journalfile,
                      struct rrdengine_datafile *datafile)
 {
-    uv_fs_t req;
     uv_file file;
     int ret, fd, error;
     uint64_t file_size, max_id;
@@ -1585,8 +1589,14 @@ int journalfile_load(struct rrdengine_instance *ctx, struct rrdengine_journalfil
         return 0;
     }
 
-    pgc_open_cache_to_journal_v2(open_cache, (Word_t) ctx, (int) datafile->fileno, ctx->config.page_type,
-                                 journalfile_migrate_to_v2_callback, (void *) datafile->journalfile);
+    pgc_open_cache_to_journal_v2(
+        open_cache,
+        (Word_t)ctx,
+        (int)datafile->fileno,
+        ctx->config.page_type,
+        journalfile_migrate_to_v2_callback,
+        (void *)datafile->journalfile,
+        true);
 
     if (is_last_file)
         ctx->loading.create_new_datafile_pair = true;
@@ -1594,11 +1604,6 @@ int journalfile_load(struct rrdengine_instance *ctx, struct rrdengine_journalfil
     return 0;
 
 cleanup:
-    ret = uv_fs_close(NULL, &req, file, NULL);
-    if (ret < 0) {
-        netdata_log_error("DBENGINE: uv_fs_close(\"%s\"): %s", path, uv_strerror(ret));
-        ctx_fs_error(ctx);
-    }
-    uv_fs_req_cleanup(&req);
+    CLOSE_FILE(ctx, path, file, ret);
     return error;
 }
