@@ -31,7 +31,7 @@ static inline void worker_dispatch_extent_read(struct rrdeng_cmd cmd, bool from_
 static inline void worker_dispatch_query_prep(struct rrdeng_cmd cmd, bool from_worker);
 
 struct rrdeng_main {
-    uv_thread_t thread;
+    ND_THREAD *thread;
     uv_loop_t loop;
     uv_async_t async;
     uv_timer_t timer;
@@ -165,16 +165,24 @@ static inline enum LIBUV_WORKERS_STATUS work_request_full(void) {
     return LIBUV_WORKERS_RELAXED;
 }
 
+// This needs to be called from event loop thread only (callback)
 static inline void check_and_schedule_db_rotation(struct rrdengine_instance *ctx)
 {
     internal_fatal(rrdeng_main.tid != gettid_cached(), "check_and_schedule_db_rotation() can only be run from the event loop thread");
 
+    if (__atomic_load_n(&ctx->atomic.needs_indexing, __ATOMIC_RELAXED)) {
+        if (ctx->datafiles.pending_index == false) {
+            ctx->datafiles.pending_index = true;
+            rrdeng_enq_cmd(ctx, RRDENG_OPCODE_JOURNAL_INDEX, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+        }
+    }
+
     if (ctx->datafiles.pending_rotate) {
         nd_log_daemon(NDLP_DEBUG, "DBENGINE: tier %d is already pending rotation", ctx->config.tier);
         return;
     }
 
-    if(ctx_is_available_for_queries(ctx) && rrdeng_ctx_tier_cap_exceeded(ctx)) {
+    if(rrdeng_ctx_tier_cap_exceeded(ctx)) {
         ctx->datafiles.pending_rotate = true;
         rrdeng_enq_cmd(ctx, RRDENG_OPCODE_DATABASE_ROTATE, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
     }
@@ -609,6 +617,7 @@ static void journalfile_extent_build(struct rrdengine_instance *ctx, struct exte
 
     df_header = xt_io_descr->buf;
     count = df_header->number_of_pages;
+    fatal_assert(count <= MAX_PAGES_PER_EXTENT);
     descr_size = sizeof(*jf_metric_data->descr) * count;
     payload_length = sizeof(*jf_metric_data) + descr_size;
     size_bytes = sizeof(*jf_header) + payload_length + sizeof(*jf_trailer);
@@ -670,8 +679,7 @@ extent_flush_to_open(struct rrdengine_instance *ctx, struct extent_io_descriptor
     spinlock_unlock(&datafile->writers.spinlock);
 
     if(datafile->fileno != ctx_last_fileno_get(ctx) && still_running)
-        // we just finished a flushing on a datafile that is not the active one
-        rrdeng_enq_cmd(ctx, RRDENG_OPCODE_JOURNAL_INDEX, datafile, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
+        __atomic_store_n(&ctx->atomic.needs_indexing, true, __ATOMIC_RELAXED);
 
     worker_is_idle();
 }
@@ -682,7 +690,7 @@ static bool datafile_is_full(struct rrdengine_instance *ctx, struct rrdengine_da
     bool ret = false;
     spinlock_lock(&datafile->writers.spinlock);
 
-    if(ctx_is_available_for_queries(ctx) && datafile->pos > rrdeng_target_data_file_size(ctx))
+    if(datafile->pos > rrdeng_target_data_file_size(ctx))
         ret = true;
 
     spinlock_unlock(&datafile->writers.spinlock);
@@ -690,12 +698,98 @@ static bool datafile_is_full(struct rrdengine_instance *ctx, struct rrdengine_da
     return ret;
 }
 
+size_t datafile_count(struct rrdengine_instance *ctx, bool with_lock)
+{
+    size_t count = 0;
+
+    if (!with_lock)
+        uv_rwlock_rdlock(&ctx->datafiles.rwlock);
+
+    count = JudyLCount(ctx->datafiles.JudyL, 0, -1, PJE0);
+
+    if (!with_lock)
+        uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
+
+    return count;
+}
+
+struct rrdengine_datafile *
+get_next_datafile(struct rrdengine_datafile *this_datafile, struct rrdengine_instance *ctx, bool with_lock)
+{
+    struct rrdengine_datafile *datafile = NULL;
+
+    ctx = this_datafile ? this_datafile->ctx : ctx;
+    if (!ctx)
+        return NULL;
+
+    if (!with_lock)
+        uv_rwlock_rdlock(&ctx->datafiles.rwlock);
+
+    Word_t Index = this_datafile ? this_datafile->fileno : 0;
+    Pvoid_t *Pvalue;
+
+    Pvalue = JudyLNext(ctx->datafiles.JudyL, &Index, PJE0);
+
+    if (Pvalue)
+        datafile = *Pvalue;
+
+    if (!with_lock)
+        uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
+
+    return datafile;
+}
+
+static struct rrdengine_datafile *get_ctx_datafile_first_or_last(struct rrdengine_instance *ctx, bool first, bool with_lock)
+{
+    struct rrdengine_datafile *datafile = NULL;
+
+    if (!with_lock)
+        uv_rwlock_rdlock(&ctx->datafiles.rwlock);
+
+    Word_t Index = 0;
+    Pvoid_t *Pvalue;
+
+    if (first)
+        Pvalue = JudyLFirst(ctx->datafiles.JudyL, &Index, PJE0);
+    else {
+        Index = -1;
+        Pvalue = JudyLLast(ctx->datafiles.JudyL, &Index, PJE0);
+    }
+
+    if (Pvalue)
+        datafile = *Pvalue;
+
+    if (!with_lock)
+        uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
+
+    return datafile;
+}
+
+struct rrdengine_datafile *get_first_ctx_datafile(struct rrdengine_instance *ctx, bool with_lock) {
+    return get_ctx_datafile_first_or_last(ctx, true, with_lock);
+}
+
+struct rrdengine_datafile *get_last_ctx_datafile(struct rrdengine_instance *ctx, bool with_lock) {
+    return get_ctx_datafile_first_or_last(ctx, false, with_lock);
+}
+
+static netdata_mutex_t mutex;
+
+static void __attribute__((constructor)) init_mutex(void) {
+    netdata_mutex_init(&mutex);
+}
+
+static void __attribute__((destructor)) destroy_mutex(void) {
+    netdata_mutex_destroy(&mutex);
+}
+
 static struct rrdengine_datafile *get_datafile_to_write_extent(struct rrdengine_instance *ctx) {
     struct rrdengine_datafile *datafile;
 
     // get the latest datafile
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-    datafile = ctx->datafiles.first->prev;
+
+    datafile = get_last_ctx_datafile(ctx, true);
     // become a writer on this datafile, to prevent it from vanishing
     spinlock_lock(&datafile->writers.spinlock);
     datafile->writers.running++;
@@ -707,23 +801,20 @@ static struct rrdengine_datafile *get_datafile_to_write_extent(struct rrdengine_
         struct rrdengine_datafile *old_datafile = datafile;
 
         // only 1 datafile creation at a time
-        static netdata_mutex_t mutex = NETDATA_MUTEX_INITIALIZER;
+
         netdata_mutex_lock(&mutex);
 
         // take the latest datafile again - without this, multiple threads may create multiple files
-        uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-        datafile = ctx->datafiles.first->prev;
-        uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
+        datafile = get_last_ctx_datafile(ctx, false);
 
-        if(datafile_is_full(ctx, datafile) && create_new_datafile_pair(ctx, true) == 0)
-            rrdeng_enq_cmd(ctx, RRDENG_OPCODE_JOURNAL_INDEX, datafile, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL,
-                           NULL);
+        if(datafile_is_full(ctx, datafile) && create_new_datafile_pair(ctx) == 0)
+            __atomic_store_n(&ctx->atomic.needs_indexing, true, __ATOMIC_RELAXED);
 
         netdata_mutex_unlock(&mutex);
 
         // get the new latest datafile again, like above
         uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-        datafile = ctx->datafiles.first->prev;
+        datafile = get_last_ctx_datafile(ctx, true);
         // become a writer on this datafile, to prevent it from vanishing
         spinlock_lock(&datafile->writers.spinlock);
         datafile->writers.running++;
@@ -871,9 +962,6 @@ datafile_extent_build(struct rrdengine_instance *ctx, struct page_descr_with_dat
 
 static void after_extent_write(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* uv_work_req __maybe_unused, int status __maybe_unused)
 {
-    if(completion)
-        completion_mark_complete(completion);
-
     check_and_schedule_db_rotation(ctx);
 }
 
@@ -928,6 +1016,7 @@ static void *extent_write_tp_worker(
 
 done:
     __atomic_sub_fetch(&ctx->atomic.extents_currently_being_flushed, 1, __ATOMIC_RELAXED);
+    completion_mark_complete(completion);
     worker_is_idle();
     return NULL;
 }
@@ -935,9 +1024,6 @@ done:
 static void after_database_rotate(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
     __atomic_store_n(&ctx->atomic.now_deleting_files, false, __ATOMIC_RELAXED);
 
-    if (__atomic_load_n(&ctx->atomic.needs_indexing, __ATOMIC_RELAXED))
-       rrdeng_enq_cmd(ctx, RRDENG_OPCODE_JOURNAL_INDEX, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
-
     check_and_schedule_db_rotation(ctx);
 }
 
@@ -954,10 +1040,10 @@ struct rrdengine_datafile *datafile_release_and_acquire_next_for_retention(struc
 
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
 
-    struct rrdengine_datafile *next_datafile = datafile->next;
+    struct rrdengine_datafile *next_datafile = get_next_datafile(datafile, NULL, true);
 
     while(next_datafile && !datafile_acquire(next_datafile, DATAFILE_ACQUIRE_RETENTION))
-        next_datafile = next_datafile->next;
+        next_datafile = get_next_datafile(next_datafile, NULL, true);
 
     uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
 
@@ -966,7 +1052,7 @@ struct rrdengine_datafile *datafile_release_and_acquire_next_for_retention(struc
     return next_datafile;
 }
 
-time_t find_uuid_first_time(
+static time_t find_uuid_first_time(
     struct rrdengine_instance *ctx,
     struct rrdengine_datafile *datafile,
     struct uuid_first_time_s *uuid_first_entry_list,
@@ -977,7 +1063,8 @@ time_t find_uuid_first_time(
     // acquire the datafile to work with it
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
     while(datafile && !datafile_acquire(datafile, DATAFILE_ACQUIRE_RETENTION))
-        datafile = datafile->next;
+        datafile = get_next_datafile(datafile, NULL, true);
+
     uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
 
     if (unlikely(!datafile))
@@ -987,6 +1074,7 @@ time_t find_uuid_first_time(
     size_t binary_match = 0;
     size_t not_matching_bsearches = 0;
 
+    bool agent_shutdown = false;
     while (datafile) {
         struct journal_v2_header *j2_header = journalfile_v2_data_acquire(datafile->journalfile, NULL, 0, 0);
         if (!j2_header) {
@@ -994,45 +1082,59 @@ time_t find_uuid_first_time(
             continue;
         }
 
-        time_t journal_start_time_s = (time_t) (j2_header->start_time_ut / USEC_PER_SEC);
+        bool any_matching = false;
 
-        if(journal_start_time_s < global_first_time_s)
+        time_t journal_start_time_s = (time_t)(j2_header->start_time_ut / USEC_PER_SEC);
+
+        if (journal_start_time_s < global_first_time_s)
             global_first_time_s = journal_start_time_s;
 
-        struct journal_metric_list *uuid_list = (struct journal_metric_list *)((uint8_t *) j2_header + j2_header->metric_offset);
+        struct journal_metric_list *uuid_list =
+            (struct journal_metric_list *)((uint8_t *)j2_header + j2_header->metric_offset);
         struct uuid_first_time_s *uuid_original_entry;
 
         size_t journal_metric_count = j2_header->metric_count;
         char file_path[RRDENG_PATH_MAX];
         journalfile_v2_generate_path(datafile, file_path, sizeof(file_path));
         PROTECTED_ACCESS_SETUP(datafile->journalfile->mmap.data, datafile->journalfile->mmap.size, file_path, "read");
-        if(no_signal_received) {
+        if (no_signal_received) {
+            size_t journal_search_start = 0; // Start of remaining search space
+            any_matching = false;
             for (size_t index = 0; index < count; ++index) {
                 uuid_original_entry = &uuid_first_entry_list[index];
 
-                // Check here if we should skip this
                 if (uuid_original_entry->df_matched > 3 || uuid_original_entry->pages_found > 5)
                     continue;
 
-                struct journal_metric_list *live_entry = bsearch(
-                    uuid_original_entry->uuid,
-                    uuid_list,
-                    journal_metric_count,
-                    sizeof(*uuid_list),
-                    journal_metric_uuid_compare);
+                any_matching = true;
+                struct journal_metric_list *live_entry = &uuid_list[journal_search_start];
+                // Check if we avoid bsearch
+                if (journal_metric_uuid_compare(uuid_original_entry->uuid, live_entry->uuid) != 0) {
+                    live_entry = bsearch(
+                        uuid_original_entry->uuid,
+                        uuid_list + journal_search_start,
+                        journal_metric_count - journal_search_start,
+                        sizeof(*uuid_list),
+                        journal_metric_uuid_compare);
 
-                if (!live_entry) {
-                    // Not found in this journal
-                    not_matching_bsearches++;
-                    continue;
+                    if (!live_entry) {
+                        not_matching_bsearches++;
+                        continue;
+                    }
+                }
+
+                size_t found_index = live_entry - uuid_list;
+                journal_search_start = found_index + 1; // Next search starts after this match
+
+                if (journal_search_start >= journal_metric_count) {
+                    not_matching_bsearches += (count - index - 1);
+                    break;
                 }
 
                 uuid_original_entry->pages_found += live_entry->entries;
                 uuid_original_entry->df_matched++;
 
                 time_t old_first_time_s = uuid_original_entry->first_time_s;
-
-                // Calculate first / last for this match
                 time_t first_time_s = live_entry->delta_start_s + journal_start_time_s;
                 uuid_original_entry->first_time_s = MIN(uuid_original_entry->first_time_s, first_time_s);
 
@@ -1040,17 +1142,34 @@ time_t find_uuid_first_time(
                     uuid_original_entry->df_index_oldest = uuid_original_entry->df_matched;
 
                 binary_match++;
+
+                if (unlikely(!ctx_is_available_for_queries(ctx))) {
+                    agent_shutdown = true;
+                    break;
+                }
             }
         } else {
-            nd_log_daemon(
-                NDLP_ERR, "DBENGINE: journalfile \"%s\" is corrupted, skipping it", file_path);
+            nd_log_daemon(NDLP_ERR, "DBENGINE: journalfile \"%s\" is corrupted, skipping it", file_path);
+        }
+        journalfile_v2_data_release(datafile->journalfile);
+
+        if (agent_shutdown) {
+            datafile_release(datafile, DATAFILE_ACQUIRE_RETENTION);
+            break;
         }
 
         journalfile_count++;
-        journalfile_v2_data_release(datafile->journalfile);
         datafile = datafile_release_and_acquire_next_for_retention(ctx, datafile);
+        if (!any_matching) {
+            if (datafile)
+                datafile_release(datafile, DATAFILE_ACQUIRE_RETENTION);
+            break;
+        }
     }
 
+    if (agent_shutdown)
+        return global_first_time_s;
+
     // Let's scan the open cache for almost exact match
     size_t open_cache_count = 0;
 
@@ -1174,15 +1293,28 @@ static void update_metrics_first_time_s(struct rrdengine_instance *ctx, struct r
 
     global_first_time_s = find_uuid_first_time(ctx, first_datafile_remaining, uuid_first_entry_list, added);
 
+    if (!ctx_is_available_for_queries(ctx)) {
+        for (size_t index = 0; index < added; ++index) {
+            uuid_first_t_entry = &uuid_first_entry_list[index];
+            mrg_metric_release(main_mrg, uuid_first_t_entry->metric);
+        }
+        goto done;
+    }
+
     if(worker)
         worker_is_busy(UV_EVENT_DBENGINE_POPULATE_MRG);
 
-    netdata_log_info("DBENGINE: updating tier %d metrics registry retention for %zu metrics",
-         ctx->config.tier, added);
+    netdata_log_info("DBENGINE: updating tier %d metrics registry retention for %zu metrics", ctx->config.tier, added);
 
     size_t deleted_metrics = 0, zero_retention_referenced = 0, zero_disk_retention = 0, zero_disk_but_live = 0;
     for (size_t index = 0; index < added; ++index) {
         uuid_first_t_entry = &uuid_first_entry_list[index];
+
+        if (!ctx_is_available_for_queries(ctx)) {
+            mrg_metric_release(main_mrg, uuid_first_t_entry->metric);
+            continue;
+        }
+
         if (likely(uuid_first_t_entry->first_time_s != LONG_MAX)) {
 
             time_t old_first_time_s = mrg_metric_get_first_time_s(main_mrg, uuid_first_t_entry->metric);
@@ -1223,7 +1355,9 @@ static void update_metrics_first_time_s(struct rrdengine_instance *ctx, struct r
             }
         }
     }
-    freez(uuid_first_entry_list);
+
+    if (!ctx_is_available_for_queries(ctx))
+        goto done;
 
     internal_error(zero_disk_retention,
                    "DBENGINE: deleted %zu metrics, zero retention but referenced %zu (out of %zu total, of which %zu have main cache retention) zero on-disk retention tier %d metrics from metrics registry",
@@ -1232,6 +1366,9 @@ static void update_metrics_first_time_s(struct rrdengine_instance *ctx, struct r
     if(global_first_time_s != LONG_MAX)
         __atomic_store_n(&ctx->atomic.first_time_s, global_first_time_s, __ATOMIC_RELAXED);
 
+done:
+    freez(uuid_first_entry_list);
+
     if(worker)
         worker_is_idle();
 }
@@ -1259,7 +1396,7 @@ void datafile_delete(
                          DATAFILE_PREFIX RRDENG_FILE_NUMBER_PRINT_TMPL DATAFILE_EXTENSION
                          "' to be available for deletion, "
                          "it is in use currently by %u users.",
-                 ctx->config.dbfiles_path, ctx->datafiles.first->tier, ctx->datafiles.first->fileno, datafile->users.lockers);
+                 ctx->config.dbfiles_path, datafile->tier, datafile->fileno, datafile->users.lockers);
 
             __atomic_add_fetch(&rrdeng_cache_efficiency_stats.datafile_deletion_spin, 1, __ATOMIC_RELAXED);
             sleep_usec(1 * USEC_PER_SEC);
@@ -1269,16 +1406,23 @@ void datafile_delete(
     netdata_log_info("DBENGINE: acquired data file \"%s/"
                      DATAFILE_PREFIX RRDENG_FILE_NUMBER_PRINT_TMPL DATAFILE_EXTENSION
                      "\" for deletion.",
-                     ctx->config.dbfiles_path, ctx->datafiles.first->tier, ctx->datafiles.first->fileno);
+                     ctx->config.dbfiles_path, datafile->tier, datafile->fileno);
 
     if (update_retention)
-        update_metrics_first_time_s(ctx, datafile, datafile->next, worker);
+        update_metrics_first_time_s(ctx, datafile, get_next_datafile(datafile, NULL, true), worker);
+
+//    if (!ctx_is_available_for_queries(ctx)) {
+//        // agent is shutting down, we cannot continue
+//        if(worker)
+//            worker_is_idle();
+//        return;
+//    }
 
     __atomic_add_fetch(&rrdeng_cache_efficiency_stats.datafile_deletion_started, 1, __ATOMIC_RELAXED);
     netdata_log_info("DBENGINE: deleting data file \"%s/"
          DATAFILE_PREFIX RRDENG_FILE_NUMBER_PRINT_TMPL DATAFILE_EXTENSION
          "\".",
-         ctx->config.dbfiles_path, ctx->datafiles.first->tier, ctx->datafiles.first->fileno);
+         ctx->config.dbfiles_path, datafile->tier, datafile->fileno);
 
     if(worker)
         worker_is_busy(UV_EVENT_DBENGINE_DATAFILE_DELETE);
@@ -1330,7 +1474,9 @@ void datafile_delete(
 }
 
 static void *database_rotate_tp_worker(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t *uv_work_req __maybe_unused) {
-    datafile_delete(ctx, ctx->datafiles.first, ctx_is_available_for_queries(ctx), true, true);
+
+    struct rrdengine_datafile *datafile = get_first_ctx_datafile(ctx, false);
+    datafile_delete(ctx, datafile, ctx_is_available_for_queries(ctx), true, true);
 
     rrdcontext_db_rotation();
 
@@ -1367,7 +1513,7 @@ static void *flush_dirty_pages_of_section_tp_worker(struct rrdengine_instance *c
 
 struct mrg_load_thread {
     int max_threads;
-    uv_thread_t thread;
+    ND_THREAD *thread;
     uv_sem_t *sem;
     int tier;
     struct rrdengine_datafile *datafile;
@@ -1378,7 +1524,7 @@ struct mrg_load_thread {
 size_t max_running_threads = 0;
 size_t running_threads = 0;
 
-void journalfile_v2_populate_retention_to_mrg_worker(void *arg)
+void *journalfile_v2_populate_retention_to_mrg_worker(void *arg)
 {
     struct mrg_load_thread *mlt = arg;
     uv_sem_wait(mlt->sem);
@@ -1402,6 +1548,7 @@ void journalfile_v2_populate_retention_to_mrg_worker(void *arg)
 
     // Signal completion - this needs to be last
     __atomic_store_n(&mlt->finished, true, __ATOMIC_RELEASE);
+    return NULL;
 }
 
 static void after_populate_mrg(struct rrdengine_instance *ctx __maybe_unused, void *data __maybe_unused, struct completion *completion __maybe_unused, uv_work_t* req __maybe_unused, int status __maybe_unused) {
@@ -1425,9 +1572,11 @@ static void *populate_mrg_tp_worker(
     int rc;
 
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
+
     size_t total_datafiles = 0;
     size_t populated_datafiles = 0;
-    for (struct rrdengine_datafile *df = ctx->datafiles.first; df; df = df->next) {
+    struct rrdengine_datafile *df = NULL;
+    while ((df = get_next_datafile(df, ctx, true))) {
         total_datafiles++;
         if (df->populate_mrg.populated)
             populated_datafiles++;
@@ -1445,16 +1594,21 @@ static void *populate_mrg_tp_worker(
 
         // find a datafile to work on
         uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-        for(datafile = ctx->datafiles.first; datafile; datafile = datafile->next) {
-            if(!spinlock_trylock(&datafile->populate_mrg.spinlock))
-                continue;
-
-            if(datafile->populate_mrg.populated) {
-                spinlock_unlock(&datafile->populate_mrg.spinlock);
+        bool first_then_next = true;
+        Pvoid_t *Pvalue =  NULL;
+        Word_t Index = 0;
+        while((Pvalue = JudyLFirstThenNext(ctx->datafiles.JudyL, &Index, &first_then_next))) {
+            datafile = *Pvalue;
+            if(!spinlock_trylock(&datafile->populate_mrg.spinlock)) {
+                datafile = NULL;
                 continue;
             }
 
-            // we have the spinlock and it is not populated
+            if(datafile->populate_mrg.populated) {
+                spinlock_unlock(&datafile->populate_mrg.spinlock);
+                datafile = NULL;
+                continue;
+            }
             break;
         }
         uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
@@ -1472,7 +1626,7 @@ static void *populate_mrg_tp_worker(
                 if (__atomic_load_n(&mlt[index].finished, __ATOMIC_RELAXED) &&
                     __atomic_load_n(&mlt[index].tier, __ATOMIC_ACQUIRE) == tier) {
 
-                    rc = uv_thread_join(&(mlt[index].thread));
+                    rc = nd_thread_join(mlt[index].thread);
                     if (rc)
                         nd_log_daemon(NDLP_WARNING, "Failed to join thread, rc = %d", rc);
 
@@ -1507,12 +1661,11 @@ static void *populate_mrg_tp_worker(
         __atomic_store_n(&mlt[thread_index].tier, tier, __ATOMIC_RELAXED);
         mlt[thread_index].datafile = datafile;
 
-        rc = uv_thread_create(&mlt[thread_index].thread,
-                              journalfile_v2_populate_retention_to_mrg_worker,
-                              &mlt[thread_index]);
+        mlt[thread_index].thread = nd_thread_create("MRGLOAD", NETDATA_THREAD_OPTION_DEFAULT, journalfile_v2_populate_retention_to_mrg_worker,
+                                                    &mlt[thread_index]);
 
-        if (rc) {
-            nd_log_daemon(NDLP_WARNING, "Failed to create thread, rc = %d", rc);
+        if (!mlt[thread_index].thread) {
+            nd_log_daemon(NDLP_WARNING, "Failed to create thread for MRG population");
             __atomic_store_n(&mlt[thread_index].busy, false, __ATOMIC_RELEASE);
             spinlock_unlock(&datafile->populate_mrg.spinlock);
         }
@@ -1532,7 +1685,7 @@ static void *populate_mrg_tp_worker(
 
                 if (__atomic_load_n(&mlt[index].finished, __ATOMIC_RELAXED)) {
                     // Thread is finished, join it
-                    rc = uv_thread_join(&(mlt[index].thread));
+                    rc = nd_thread_join((mlt[index].thread));
                     if (rc)
                         nd_log_daemon(NDLP_WARNING, "Failed to join thread, rc = %d", rc);
 
@@ -1643,7 +1796,7 @@ time_t get_datafile_end_time(struct rrdengine_instance *ctx)
     time_t last_time_s = 0;
 
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-    struct rrdengine_datafile *datafile = ctx->datafiles.first;
+    struct rrdengine_datafile *datafile = get_last_ctx_datafile(ctx, true);
 
     if (datafile) {
         last_time_s = datafile->journalfile->v2.last_time_s;
@@ -1709,23 +1862,32 @@ static struct rrdengine_datafile *release_and_aquire_next_datafile_for_indexing(
 
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
     if (release_datafile) {
-        datafile = release_datafile->next;
+        datafile = get_next_datafile(release_datafile, NULL, true);
         datafile_release(release_datafile, DATAFILE_ACQUIRE_INDEXING);
     }
     else
-        datafile = ctx->datafiles.first;
+        datafile = get_first_ctx_datafile(ctx, true);
 
     while (datafile && datafile->fileno != ctx_last_fileno_get(ctx) && datafile->fileno != ctx_last_flush_fileno_get(ctx)) {
         if(journalfile_v2_data_available(datafile->journalfile)) {
-            datafile = datafile->next;
+            datafile = get_next_datafile(datafile, NULL, true);
             continue;
         }
-        bool locked = datafile_acquire(datafile, DATAFILE_ACQUIRE_INDEXING);
+
+        int retries = 5;
+        bool locked = false;
+        while (retries-- > 0) {
+            locked = datafile_acquire(datafile, DATAFILE_ACQUIRE_INDEXING);
+            if (locked)
+                break;
+            sleep_usec(200 * USEC_PER_MS);
+        }
         if (locked) {
             uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
             return datafile;
         }
-        nd_log_daemon(NDLP_INFO, "DBENGINE: Datafile %u CANNOT be locked for indexing; skipping", datafile->fileno);
+        nd_log_daemon(NDLP_INFO, "DBENGINE: Datafile %u CANNOT be locked for indexing after retries; skipping", datafile->fileno);
+        datafile = get_next_datafile(datafile, NULL, true);
     }
     uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
     return NULL;
@@ -1735,6 +1897,9 @@ static struct rrdengine_datafile *release_and_aquire_next_datafile_for_indexing(
 static void *journal_v2_indexing_tp_worker(struct rrdengine_instance *ctx, void *data, struct completion *completion __maybe_unused, uv_work_t *uv_work_req __maybe_unused) {
     unsigned count = 0;
 
+    if (unlikely(!ctx_is_available_for_queries(ctx)))
+        return data;
+
     worker_is_busy(UV_EVENT_DBENGINE_JOURNAL_INDEX);
     struct rrdengine_datafile *datafile = NULL;
     char path[RRDENG_PATH_MAX];
@@ -1765,8 +1930,14 @@ static void *journal_v2_indexing_tp_worker(struct rrdengine_instance *ctx, void
         }
         nd_log_daemon(NDLP_INFO, "DBENGINE: journal file \"%s\" is ready to be indexed", path);
 
-        pgc_open_cache_to_journal_v2(open_cache, (Word_t) ctx, (int) datafile->fileno, ctx->config.page_type,
-                                     journalfile_migrate_to_v2_callback, (void *) datafile->journalfile);
+        pgc_open_cache_to_journal_v2(
+            open_cache,
+            (Word_t)ctx,
+            (int)datafile->fileno,
+            ctx->config.page_type,
+            journalfile_migrate_to_v2_callback,
+            (void *)datafile->journalfile,
+            false);
 
         index_once = true;
 
@@ -1863,8 +2034,11 @@ uint64_t rrdeng_get_used_disk_space(struct rrdengine_instance *ctx, bool having_
     if (!having_lock)
         uv_rwlock_rdlock(&ctx->datafiles.rwlock);
 
-    if (ctx->datafiles.first && ctx->datafiles.first->prev)
-        active_space = ctx->datafiles.first->prev->pos;
+    struct rrdengine_datafile *first_datafile = get_first_ctx_datafile(ctx, true);
+    struct rrdengine_datafile *last_datafile = get_last_ctx_datafile(ctx, true);
+
+    if (first_datafile && last_datafile)
+        active_space = last_datafile->pos;
 
     if (!having_lock)
         uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
@@ -1899,7 +2073,9 @@ static time_t get_tier_retention(struct rrdengine_instance *ctx)
 bool rrdeng_ctx_tier_cap_exceeded(struct rrdengine_instance *ctx)
 {
     uv_rwlock_rdlock(&ctx->datafiles.rwlock);
-    if (!ctx->datafiles.first || !ctx->datafiles.first->next) {
+    struct rrdengine_datafile *first_datafile = get_first_ctx_datafile(ctx, true);
+
+    if (!first_datafile || datafile_count(ctx, true) < 2) {
         uv_rwlock_rdunlock(&ctx->datafiles.rwlock);
         return false;
     }
@@ -2023,11 +2199,9 @@ bool rrdeng_dbengine_spawn(struct rrdengine_instance *ctx __maybe_unused) {
         dbengine_initialize_structures();
 
         int retries = 0;
-        int create_uv_thread_rc = create_uv_thread(&rrdeng_main.thread, dbengine_event_loop, &rrdeng_main, &retries);
-        if (create_uv_thread_rc)
-            nd_log_daemon(NDLP_ERR, "Failed to create DBENGINE thread, error %s, after %d retries", uv_err_name(create_uv_thread_rc), retries);
+        rrdeng_main.thread = nd_thread_create("DBEV", NETDATA_THREAD_OPTION_DEFAULT, dbengine_event_loop, &rrdeng_main);
 
-        fatal_assert(0 == create_uv_thread_rc);
+        fatal_assert(0 != rrdeng_main.thread);
 
         if (retries)
             nd_log_daemon(NDLP_WARNING, "DBENGINE thread was created after %d attempts", retries);
@@ -2103,10 +2277,10 @@ void rrdeng_calculate_tier_disk_space_percentage(void)
 #define NOT_INDEXING_FILES(ctx)                                                                                        \
     (!__atomic_load_n(&(ctx)->atomic.migration_to_v2_running, __ATOMIC_RELAXED))
 
-void dbengine_event_loop(void* arg) {
+void *dbengine_event_loop(void* arg) {
     sanity_check();
     uv_thread_set_name_np("DBENGINE");
-    service_register(SERVICE_THREAD_TYPE_EVENT_LOOP, NULL, NULL, NULL, true);
+    service_register(NULL, NULL, NULL);
 
     worker_register("DBENGINE");
 
@@ -2122,6 +2296,7 @@ void dbengine_event_loop(void* arg) {
     worker_register_job_name(RRDENG_OPCODE_EVICT_MAIN,                               "evict init");
     worker_register_job_name(RRDENG_OPCODE_CTX_SHUTDOWN,                             "ctx shutdown");
     worker_register_job_name(RRDENG_OPCODE_CTX_FLUSH_DIRTY,                          "ctx flush dirty");
+    worker_register_job_name(RRDENG_OPCODE_CTX_FLUSH_HOT_DIRTY,                      "ctx flush all");
     worker_register_job_name(RRDENG_OPCODE_CTX_QUIESCE,                              "ctx quiesce");
     worker_register_job_name(RRDENG_OPCODE_SHUTDOWN_EVLOOP,                          "dbengine shutdown");
 
@@ -2246,6 +2421,7 @@ void dbengine_event_loop(void* arg) {
                 case RRDENG_OPCODE_JOURNAL_INDEX: {
                     struct rrdengine_instance *ctx = cmd.ctx;
                     struct rrdengine_datafile *datafile = cmd.data;
+                    // We no longer have an indexing command pending
                     ctx->datafiles.pending_index = false;
                     if (NOT_INDEXING_FILES(ctx) && ctx_is_available_for_queries(ctx)) {
                         __atomic_store_n(&ctx->atomic.migration_to_v2_running, true, __ATOMIC_RELAXED);
@@ -2258,8 +2434,8 @@ void dbengine_event_loop(void* arg) {
                 case RRDENG_OPCODE_DATABASE_ROTATE: {
                     struct rrdengine_instance *ctx = cmd.ctx;
                     ctx->datafiles.pending_rotate = false;
-                    if (NOT_DELETING_FILES(ctx) && ctx->datafiles.first->next != NULL &&
-                        ctx->datafiles.first->next->next != NULL && rrdeng_ctx_tier_cap_exceeded(ctx)) {
+                    if (NOT_DELETING_FILES(ctx) && datafile_count(ctx, false) > 2 &&
+                        rrdeng_ctx_tier_cap_exceeded(ctx)) {
                         __atomic_store_n(&ctx->atomic.now_deleting_files, true, __ATOMIC_RELAXED);
                         work_dispatch(ctx, NULL, NULL, opcode, database_rotate_tp_worker, after_database_rotate);
                     }
@@ -2281,13 +2457,19 @@ void dbengine_event_loop(void* arg) {
                     break;
                 }
 
+                case RRDENG_OPCODE_CTX_FLUSH_HOT_DIRTY: {
+                    struct rrdengine_instance *ctx = cmd.ctx;
+                    work_dispatch(ctx, NULL, NULL, opcode,
+                                  flush_all_hot_and_dirty_pages_of_section_tp_worker,
+                                  after_flush_all_hot_and_dirty_pages_of_section);
+                    break;
+                }
+
                 case RRDENG_OPCODE_CTX_QUIESCE: {
                     // a ctx will shutdown shortly
                     struct rrdengine_instance *ctx = cmd.ctx;
+                    nd_log_daemon(NDLP_INFO, "DBENGINE: Tier %d is shutting down — query processing disabled", ctx->config.tier);
                     __atomic_store_n(&ctx->quiesce.enabled, true, __ATOMIC_RELEASE);
-                    work_dispatch(ctx, NULL, NULL, opcode,
-                                      flush_all_hot_and_dirty_pages_of_section_tp_worker,
-                                      after_flush_all_hot_and_dirty_pages_of_section);
                     break;
                 }
 
@@ -2332,13 +2514,14 @@ void dbengine_event_loop(void* arg) {
     nd_log(NDLS_DAEMON, NDLP_DEBUG, "Shutting down dbengine thread");
     (void) uv_loop_close(&main->loop);
     worker_unregister();
+    return NULL;
 }
 
 void dbengine_shutdown()
 {
     rrdeng_enq_cmd(NULL, RRDENG_OPCODE_SHUTDOWN_EVLOOP, NULL, NULL, STORAGE_PRIORITY_INTERNAL_DBENGINE, NULL, NULL);
 
-    int rc = uv_thread_join(&rrdeng_main.thread);
+    int rc = nd_thread_join(rrdeng_main.thread);
     if (rc)
         nd_log_daemon(NDLP_ERR, "DBENGINE: Failed to join thread, error %s", uv_err_name(rc));
     else
