@@ -35,6 +35,17 @@ extern unsigned rrdeng_pages_per_extent;
         uv_fs_req_cleanup(&(_req));                                                                                   \
     } while (0)
 
+#define CLOSE_FILE(ctx, path, file, ret_var)                                                                           \
+    do {                                                                                                               \
+        uv_fs_t _req;                                                                                                  \
+        (ret_var) = uv_fs_close(NULL, &(_req), (file), NULL);                                                          \
+        if ((ret_var) < 0) {                                                                                           \
+            netdata_log_error("DBENGINE: uv_fs_close(\"%s\"): %s", (path), uv_strerror(ret_var));                      \
+            ctx_fs_error(ctx);                                                                                         \
+        }                                                                                                              \
+        uv_fs_req_cleanup(&(_req));                                                                                    \
+    } while (0)
+
 /* Forward declarations */
 struct rrdengine_instance;
 struct rrdeng_cmd;
@@ -269,6 +280,7 @@ enum rrdeng_opcode {
     RRDENG_OPCODE_EVICT_EXTENT,
     RRDENG_OPCODE_CTX_SHUTDOWN,
     RRDENG_OPCODE_CTX_FLUSH_DIRTY,
+    RRDENG_OPCODE_CTX_FLUSH_HOT_DIRTY,
     RRDENG_OPCODE_CTX_QUIESCE,
     RRDENG_OPCODE_CTX_POPULATE_MRG,
     RRDENG_OPCODE_SHUTDOWN_EVLOOP,
@@ -385,11 +397,11 @@ struct rrdengine_instance {
     TIER_CONFIG_PROTOTYPE config;
 
     struct {
-        uv_rwlock_t rwlock;                         // the linked list of datafiles is protected by this lock
+        uv_rwlock_t rwlock;                         // the JudyL of datafiles is protected by this lock
         bool disk_time;                             // true: delete for disk quota, false: delete for retention
-        bool pending_rotate;
-        bool pending_index;
-        struct rrdengine_datafile *first;           // oldest - the newest with ->first->prev
+        bool pending_rotate;                        // Change from event loop
+        bool pending_index;                         // Change from event loop
+        Pvoid_t JudyL;                              // the datafiles, indexed by fileno
     } datafiles;
 
     struct {
@@ -424,11 +436,7 @@ struct rrdengine_instance {
     } quiesce;
 
     struct {
-        struct {
-            size_t size;
-            struct completion *array;
-        } populate_mrg;
-
+        struct completion load_mrg;
         bool create_new_datafile_pair;
     } loading;
 
@@ -479,7 +487,7 @@ bool rrdeng_ctx_tier_cap_exceeded(struct rrdengine_instance *ctx);
 int init_rrd_files(struct rrdengine_instance *ctx);
 void finalize_rrd_files(struct rrdengine_instance *ctx);
 bool rrdeng_dbengine_spawn(struct rrdengine_instance *ctx);
-void dbengine_event_loop(void *arg);
+void *dbengine_event_loop(void *arg);
 
 typedef void (*enqueue_callback_t)(struct rrdeng_cmd *cmd);
 typedef void (*dequeue_callback_t)(struct rrdeng_cmd *cmd);
@@ -573,5 +581,11 @@ uint64_t rrdeng_get_used_disk_space(struct rrdengine_instance *ctx, bool having_
 void rrdeng_calculate_tier_disk_space_percentage(void);
 uint64_t rrdeng_get_directory_free_bytes_space(struct rrdengine_instance *ctx);
 void dbengine_shutdown();
+size_t datafile_count(struct rrdengine_instance *ctx, bool with_lock);
+struct rrdengine_datafile *get_first_ctx_datafile(struct rrdengine_instance *ctx, bool with_lock);
+struct rrdengine_datafile *get_last_ctx_datafile(struct rrdengine_instance *ctx, bool with_lock);
+struct rrdengine_datafile *
+get_next_datafile(struct rrdengine_datafile *this_datafile, struct rrdengine_instance *ctx, bool with_lock);
+
 
 #endif /* NETDATA_RRDENGINE_H */
