@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 package journaldexporter
 
 import (
@@ -25,7 +27,8 @@ func createDefaultConfig() component.Config {
 }
 
 func createLogsExporter(ctx context.Context, set exporter.Settings, cfg component.Config) (exporter.Logs, error) {
-	exp := newMyExporter()
+	exp := newJournaldExporter(cfg, set.Logger)
+
 	return exporterhelper.NewLogs(
 		ctx,
 		set,
