@@ -62,19 +62,22 @@ type (
 		profile        *ddsnmp.Profile
 		initialized    bool
 		globalTags     map[string]string
-		deviceMetadata map[string]string
+		deviceMetadata map[string]ddsnmp.MetaTag
 	}
 )
 
-func (c *Collector) CollectDeviceMetadata() (map[string]map[string]string, error) {
-	meta := make(map[string]map[string]string)
+func (c *Collector) CollectDeviceMetadata() (map[string]ddsnmp.MetaTag, error) {
+	meta := make(map[string]ddsnmp.MetaTag)
 
 	for _, prof := range c.profiles {
-		deviceMeta, err := c.deviceMetadataCollector.Collect(prof.profile)
+		profDeviceMeta, err := c.deviceMetadataCollector.Collect(prof.profile)
 		if err != nil {
 			return nil, err
 		}
-		meta[prof.profile.SourceFile] = deviceMeta
+
+		for k, v := range profDeviceMeta {
+			mergeMetaTagIfAbsent(meta, k, v)
+		}
 	}
 
 	return meta, nil
