@@ -6,7 +6,7 @@ import (
 
 type ProfileMetrics struct {
 	Source         string
-	DeviceMetadata map[string]string
+	DeviceMetadata map[string]MetaTag
 	Tags           map[string]string
 	Metrics        []Metric
 }
@@ -26,3 +26,8 @@ type Metric struct {
 
 	IsTable bool
 }
+
+type MetaTag struct {
+	Value        string
+	IsExactMatch bool // whether this value is from an exact match context
+}
