@@ -24,88 +24,121 @@ static SOCKET_PEERS netdata_ssl_peers(NETDATA_SSL *ssl) {
 static void netdata_ssl_log_error_queue(const char *call, NETDATA_SSL *ssl, unsigned long err) {
     nd_log_limit_static_thread_var(erl, 1, 0);
 
-    if(err == SSL_ERROR_NONE)
+    if (err == SSL_ERROR_NONE)
         err = ERR_get_error();
 
-    if(err == SSL_ERROR_NONE)
+    if (err == SSL_ERROR_NONE)
         return;
 
-    do {
-        char *code;
+    SOCKET_PEERS peers = netdata_ssl_peers(ssl);
+    const char *ssl_state = ssl->conn ? SSL_state_string_long(ssl->conn) : "No SSL connection";
+    const char *cipher = ssl->conn ? SSL_get_cipher(ssl->conn) : "Unknown";
+    const char *alpn_proto = NULL;
+    unsigned int alpn_len = 0;
 
+#ifdef OPENSSL_NPN_NEGOTIATED
+    SSL_get0_alpn_selected(ssl->conn, (const unsigned char **)&alpn_proto, &alpn_len);
+#endif
+
+    do {
+        char *err_code;
         switch (err) {
             case SSL_ERROR_SSL:
-                code = "SSL_ERROR_SSL";
+                err_code = "SSL_ERROR_SSL";
                 ssl->state = NETDATA_SSL_STATE_FAILED;
                 break;
 
             case SSL_ERROR_WANT_READ:
-                code = "SSL_ERROR_WANT_READ";
+                err_code = "SSL_ERROR_WANT_READ";
                 break;
 
             case SSL_ERROR_WANT_WRITE:
-                code = "SSL_ERROR_WANT_WRITE";
+                err_code = "SSL_ERROR_WANT_WRITE";
                 break;
 
             case SSL_ERROR_WANT_X509_LOOKUP:
-                code = "SSL_ERROR_WANT_X509_LOOKUP";
+                err_code = "SSL_ERROR_WANT_X509_LOOKUP";
                 break;
 
             case SSL_ERROR_SYSCALL:
-                code = "SSL_ERROR_SYSCALL";
+                err_code = "SSL_ERROR_SYSCALL";
                 ssl->state = NETDATA_SSL_STATE_FAILED;
                 break;
 
             case SSL_ERROR_ZERO_RETURN:
-                code = "SSL_ERROR_ZERO_RETURN";
+                err_code = "SSL_ERROR_ZERO_RETURN";
+                ssl->state = NETDATA_SSL_STATE_FAILED;
                 break;
 
             case SSL_ERROR_WANT_CONNECT:
-                code = "SSL_ERROR_WANT_CONNECT";
+                err_code = "SSL_ERROR_WANT_CONNECT";
                 break;
 
             case SSL_ERROR_WANT_ACCEPT:
-                code = "SSL_ERROR_WANT_ACCEPT";
+                err_code = "SSL_ERROR_WANT_ACCEPT";
                 break;
 
 #ifdef SSL_ERROR_WANT_ASYNC
             case SSL_ERROR_WANT_ASYNC:
-                code = "SSL_ERROR_WANT_ASYNC";
+                err_code = "SSL_ERROR_WANT_ASYNC";
                 break;
 #endif
 
 #ifdef SSL_ERROR_WANT_ASYNC_JOB
             case SSL_ERROR_WANT_ASYNC_JOB:
-                code = "SSL_ERROR_WANT_ASYNC_JOB";
+                err_code = "SSL_ERROR_WANT_ASYNC_JOB";
                 break;
 #endif
 
 #ifdef SSL_ERROR_WANT_CLIENT_HELLO_CB
             case SSL_ERROR_WANT_CLIENT_HELLO_CB:
-                code = "SSL_ERROR_WANT_CLIENT_HELLO_CB";
+                err_code = "SSL_ERROR_WANT_CLIENT_HELLO_CB";
                 break;
 #endif
 
 #ifdef SSL_ERROR_WANT_RETRY_VERIFY
             case SSL_ERROR_WANT_RETRY_VERIFY:
-                code = "SSL_ERROR_WANT_RETRY_VERIFY";
+                err_code = "SSL_ERROR_WANT_RETRY_VERIFY";
                 break;
 #endif
 
             default:
-                code = "SSL_ERROR_UNKNOWN";
+                err_code = "SSL_ERROR_UNKNOWN";
                 break;
         }
 
-        char str[1024 + 1];
-        ERR_error_string_n(err, str, 1024);
-        str[1024] = '\0';
-        SOCKET_PEERS peers = netdata_ssl_peers(ssl);
-        nd_log_limit(&erl, NDLS_DAEMON, NDLP_ERR,
-                     "SSL: %s() on socket local [[%s]:%d] <-> remote [[%s]:%d], returned error %lu (%s): %s",
-                     call, peers.local.ip, peers.local.port, peers.peer.ip, peers.peer.port, err, code, str);
+        const char *reason = ERR_reason_error_string(err);
+        int reason_code = ERR_GET_REASON(err);
 
-    } while((err = ERR_get_error()));
+        char err_str[1024 + 1];
+        ERR_error_string_n(err, err_str, 1024);
+
+        // Extract TLS Alert Information
+        const char *alert_type = "None";
+        const char *alert_desc = "None";
+
+        if (ERR_GET_LIB(err) == ERR_LIB_SSL) {  // Ensure it's an SSL error
+            alert_type = SSL_alert_type_string_long(reason_code);
+            alert_desc = SSL_alert_desc_string_long(reason_code);
+        }
+
+        nd_log_limit(&erl, NDLS_DAEMON, NDLP_ERR,
+                     "SSL ERROR: %s() on socket "
+                     "local [[%s]:%d] <-> remote [[%s]:%d], "
+                     "State [%s], Cipher: [%s], ALPN: [%.*s], "
+                     "Error [%lu, %s, %s], "
+                     "Reason [%d, %s], "
+                     "Alert [%s, %s], "
+                     "Errno [%d]",
+                     call,
+                     peers.local.ip, peers.local.port, peers.peer.ip, peers.peer.port,
+                     ssl_state, cipher, (int)alpn_len, alpn_proto ? alpn_proto : "None",
+                     err, err_code, err_str,
+                     reason_code, reason ? reason : "Unknown",
+                     alert_type, alert_desc,
+                     errno);
+
+    } while ((err = ERR_get_error()));
 }
 
 bool netdata_ssl_open_ext(NETDATA_SSL *ssl, SSL_CTX *ctx, int fd, const unsigned char *alpn_protos, unsigned int alpn_protos_len) {
@@ -158,6 +191,7 @@ bool netdata_ssl_open(NETDATA_SSL *ssl, SSL_CTX *ctx, int fd) {
     return netdata_ssl_open_ext(ssl, ctx, fd, NULL, 0);
 }
 
+ALWAYS_INLINE
 void netdata_ssl_close(NETDATA_SSL *ssl) {
     errno = 0;
     ssl->ssl_errno = 0;
@@ -177,7 +211,8 @@ void netdata_ssl_close(NETDATA_SSL *ssl) {
     *ssl = NETDATA_SSL_UNSET_CONNECTION;
 }
 
-static inline bool is_handshake_complete(NETDATA_SSL *ssl, const char *op) {
+ALWAYS_INLINE
+static bool is_handshake_complete(NETDATA_SSL *ssl, const char *op) {
     nd_log_limit_static_thread_var(erl, 1, 0);
 
     if(unlikely(!ssl->conn)) {
@@ -231,10 +266,12 @@ static inline bool is_handshake_complete(NETDATA_SSL *ssl, const char *op) {
  *     (These are often the same value, but can be different on some systems.)
  */
 
+ALWAYS_INLINE
 ssize_t netdata_ssl_pending(NETDATA_SSL *ssl) {
     return SSL_pending(ssl->conn);
 }
 
+ALWAYS_INLINE
 bool netdata_ssl_has_pending(NETDATA_SSL *ssl) {
     // this call was added on OpenSSL 1.1.0
     // however, it is more accurate than SSL_pending()
@@ -244,6 +281,7 @@ bool netdata_ssl_has_pending(NETDATA_SSL *ssl) {
     return SSL_pending(ssl->conn) > 0;
 }
 
+ALWAYS_INLINE
 ssize_t netdata_ssl_read(NETDATA_SSL *ssl, void *buf, size_t num) {
     errno = 0;
     ssl->ssl_errno = 0;
@@ -286,6 +324,7 @@ ssize_t netdata_ssl_read(NETDATA_SSL *ssl, void *buf, size_t num) {
  *     (These are often the same value, but can be different on some systems.)
  */
 
+ALWAYS_INLINE
 ssize_t netdata_ssl_write(NETDATA_SSL *ssl, const void *buf, size_t num) {
     errno = 0;
     ssl->ssl_errno = 0;
@@ -353,7 +392,8 @@ static inline bool is_handshake_initialized(NETDATA_SSL *ssl, const char *op) {
 
 #define WANT_READ_WRITE_TIMEOUT_MS 10
 
-static inline bool want_read_write_should_retry(NETDATA_SSL *ssl, int err) {
+ALWAYS_INLINE
+static bool want_read_write_should_retry(NETDATA_SSL *ssl, int err) {
     int ssl_errno = SSL_get_error(ssl->conn, err);
     if(ssl_errno == SSL_ERROR_WANT_READ || ssl_errno == SSL_ERROR_WANT_WRITE) {
         struct pollfd pfds[1] = { [0] = {
