@@ -10,10 +10,10 @@
 extern "C" {
 #endif
 
-#define CBUFFER_INITIAL_SIZE (16 * 1024)
-#define CBUFFER_INITIAL_MAX_SIZE (10 * 1024 * 1024)
-#define HOST_THREAD_BUFFER_INITIAL_SIZE (256 * 1024)
-#define REPLICATION_THREAD_BUFFER_INITIAL_SIZE (512 * 1024)
+#define CBUFFER_INITIAL_SIZE (16ULL * 1024)
+#define CBUFFER_INITIAL_MAX_SIZE (10ULL * 1024 * 1024)
+#define HOST_THREAD_BUFFER_INITIAL_SIZE (256ULL * 1024)
+#define REPLICATION_THREAD_BUFFER_INITIAL_SIZE (512ULL * 1024)
 
 #define STREAM_CIRCULAR_BUFFER_ADAPT_TO_TIMES_MAX_SIZE 3
 
