@@ -137,6 +137,7 @@ static int stream_receiver_response_too_busy_now(struct web_client *w) {
 }
 
 static void stream_receiver_takeover_web_connection(struct web_client *w, struct receiver_state *rpt) {
+    // Set the file descriptor and ssl from the web client
     rpt->sock.fd = w->fd;
     rpt->sock.ssl = w->ssl;
 
@@ -151,6 +152,8 @@ static void stream_receiver_takeover_web_connection(struct web_client *w, struct
         w->fd = -1;
 
     buffer_flush(w->response.data);
+
+    web_server_remove_current_socket_from_poll();
 }
 
 static void stream_send_error_on_taken_over_connection(struct receiver_state *rpt, const char *msg) {
@@ -339,7 +342,7 @@ int stream_receiver_accept_connection(struct web_client *w, char *decoded_query_
     rrdhost_system_info_hops_set(rpt->system_info, rpt->hops);
 
     nd_sock_init(&rpt->sock, netdata_ssl_web_server_ctx, false);
-    rpt->remote_ip = strdupz(w->client_ip);
+    rpt->remote_ip = strdupz(w->user_auth.client_ip);
     rpt->remote_port = strdupz(w->client_port);
 
     rpt->config.update_every = nd_profile.update_every;
@@ -527,7 +530,7 @@ int stream_receiver_accept_connection(struct web_client *w, char *decoded_query_
         return stream_receiver_response_permission_denied(w);
     }
 
-    if(!stream_conf_api_key_allows_client(rpt->key, w->client_ip)) {
+    if(!stream_conf_api_key_allows_client(rpt->key, w->user_auth.client_ip)) {
         stream_receiver_log_status(
             rpt,
             "rejecting streaming connection; API key is not allowed from this IP",
@@ -559,7 +562,7 @@ int stream_receiver_accept_connection(struct web_client *w, char *decoded_query_
         return stream_receiver_response_permission_denied(w);
     }
 
-    if(!stream_conf_api_key_allows_client(rpt->machine_guid, w->client_ip)) {
+    if(!stream_conf_api_key_allows_client(rpt->machine_guid, w->user_auth.client_ip)) {
         stream_receiver_log_status(
             rpt,
             "rejecting streaming connection; machine UUID is not allowed from this IP",
